/* idrslib.c                                 */
/* Interferometry Data Reduction Software    */
/* by Stefan Kraus                           */
/* 2002-05                                   */
/*                                           */
/* copyright laws apply                      */


#include "string.h"
#include "bswap.c"
#include "stdio.h"
#include <sys/stat.h>
#include "ludcmp.c"
#include "lubksb.c"

void swap_bytes(int *d)
{
  M4Byte temp;

  temp.i4d = *d;
  swap4(&temp);
  *d = temp.i4d;
}

static int CreatePath2(char *Path)
{
   int          Len;
   char        *Ptr1;
   char        *Ptr2;

   if (access(Path, 00) == 0)
      return 1;

   Len = (int) strlen(Path) - 1;
   while (Len >= 0 && (Path[Len] == '\\' || Path[Len] == '/'))
      Path[Len--] = '\0';

   Ptr1 = strrchr(Path, '\\');
   Ptr2 = strrchr(Path, '/');

   if (Ptr1 && (!Ptr2 || Ptr2 < Ptr1))
      *Ptr1 = '\0';
   else if (Ptr2 && (!Ptr1 || Ptr1 < Ptr2))
      *Ptr2 = '\0';
   else
      return 0;

   if (!CreatePath(Path))
      return 0;
   return 2;
}


int CreatePath(const char *Path)
{
   char        *Path2;
   int          Res;

   Path2 = strdup(Path);
   Res = CreatePath2(Path2);
   free(Path2);

   switch (Res)
   {
    case 0:
      /* error */
      return 0;
    case 1:
      /* directory exists */
      break;
    case 2:
      /* directory doesn't exist */
      if (mkdir(Path, S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IWGRP ) != 0)
         return 0;
      break;
   }

   return 1;
}


int file_exist(char *file_name) 
{
      struct stat file_stat;
      return ( (stat(file_name, &file_stat) == 0) ? 1:0);
}


fftw_complex cmult(fftw_complex x, fftw_complex y)
{
    fftw_complex c;

    c.re = x.re*y.re - x.im*y.im;
    c.im = x.re*y.im + x.im*y.re;

    return c;
}



void smooth_signal(double *data, int n, float smoothfactor, fftw_plan fftw_f, fftw_plan fftw_b, double *data_s)
{
    int i, k;

    double norm_pixel;
    double norm_gauss;

    double norm_in, norm_out;

    fftw_complex scan[3*n], gauss[3*n], scan_smooth[3*n];
    fftw_complex ftscan[3*n], ftgauss[3*n];


    //*** Generate Gaussian for smoothing
    norm_gauss=0.0;
    for (i=0; i<3*n; i++)
    {
	//very narrow: 256, very width: 1
	gauss[i].re = exp(-log(2.0)*dsqr((double)(i-n/2)*2.0*(1.0 - smoothfactor)));
	gauss[i].im = 0.0;

	norm_gauss += gauss[i].re;
    }


    fftw_one(fftw_f, gauss, ftgauss);

    //*** Apply sky-subtraction and smoothing
    norm_pixel = 0.0;
    for (i=0; i<n; i++)
    {
	norm_pixel += data[i];
	
	scan[i+n].re = data[i];
	scan[i+n].im = 0.0;
	scan[i].re = (data[0]+data[1]+data[2]+data[3]+data[4])/5.0;
	scan[i].im = 0.0;
	scan[i+2*n].re = (data[n-1]+data[n-2]+data[n-3]+data[n-4]+data[n-5])/5.0;
	scan[i+2*n].im = 0.0;
    }
    
    //smooth signal
    fftw_one(fftw_f, scan, ftscan);

    for (i=0; i<3*n; i++)
	ftscan[i] = cmult(ftscan[i], ftgauss[i]);
    
    fftw_one(fftw_b, ftscan, scan_smooth);
    
    norm_in = 0.0;
    norm_out = 0.0;

    for (i=0; i<n; i++)
    {
	k = i-n/2+n;
	if (k < 0) k += n;
	
	data_s[i] = scan_smooth[k+n].re/(norm_gauss*3.0*(float)n);

	norm_in += data[i];
	norm_out += data_s[i];
    }
}
    

void smooth_signal_float(float *data, int n, float smoothfactor, fftw_plan fftw_f, fftw_plan fftw_b, float *data_s)
{
    int i, k;

    double norm_pixel;
    double norm_gauss;

    fftw_complex scan[3*n], gauss[3*n], scan_smooth[3*n];
    fftw_complex ftscan[3*n], ftgauss[3*n];


    //*** Generate Gaussian for smoothing
    norm_gauss=0.0;
    for (i=0; i<3*n; i++)
    {
	//very narrow: 256, very width: 1
	gauss[i].re = exp(-log(2.0)*dsqr((double)(i-n/2)*2.0*(1.0 - smoothfactor)));
	gauss[i].im = 0.0;

	norm_gauss += gauss[i].re;
    }

    fftw_one(fftw_f, gauss, ftgauss);
    
    //for (i=0; i<n; i++)
    //printf("%i %lf\n", i, gauss[i].re);

    //*** Apply sky-subtraction and smoothing
    norm_pixel = 0.0;
    for (i=0; i<n; i++)
    {
	norm_pixel += data[i];
	
	scan[i+n].re = data[i];
	scan[i+n].im = 0.0;
	scan[i].re = (data[0]+data[1]+data[2]+data[3]+data[4])/5.0;
	scan[i].im = 0.0;
	scan[i+2*n].re = (data[n-1]+data[n-2]+data[n-3]+data[n-4]+data[n-5])/5.0;
	scan[i+2*n].im = 0.0;
    }
    
    //smooth signal
    fftw_one(fftw_f, scan, ftscan);
    
    for (i=0; i<3*n; i++)
	ftscan[i] = cmult(ftscan[i], ftgauss[i]);
    
    fftw_one(fftw_b, ftscan, scan_smooth);
    
    for (i=0; i<n; i++)
    {
	k = i-n/2+n;
	if (k < 0) k += n;
	
	data_s[i] = scan_smooth[k+n].re/(norm_gauss*3.0*(float)n);
    }
}
    



//calculates the average for a double-array data[0..number]

void average(int number, float *data, double *mean, double *variance)
{
  int i, count;
  double sum;

  sum = 0;
  count = 0;
  for (i=0; i<=number; i++)
      if (data[i] >= 0.0)
      {
	  sum += data[i];
	  count++;
      }

  *mean = (sum/(count+1));

  *variance = 0.0;
  for (i=0; i<=number; i++)
      if (data[i] >= 0.0)
      {
	  *variance += fsqr(data[i]-*mean);
      }	  

  *variance = sqrt(*variance/count);
}


//calculates the average for a double-array data[0..number], limited by a specific range

void average_certain_range(int number, float *data, double *mean, double *variance, double min, double max)
{
  int i, count;
  double sum;

  sum = 0;
  count = 0;
  for (i=0; i<=number; i++)
      if ((data[i] >= min) && ((data[i] <= max)))
      {
	  sum += data[i];
	  count++;
      }

  *mean = (sum/(count+1));

  *variance = 0.0;
  for (i=0; i<=number; i++)
      if ((data[i] >= min) && ((data[i] <= max)))
      {
	  *variance += fsqr(data[i]-*mean);
      }	  

  *variance = sqrt(*variance/count);
}


//calculates the median for a double-array data[0..number]

double median(int number, float *data)
{
  float *sortdata;
  double med;
  int i, numberhalf;

  sortdata = vector(1, number+1);
 
  for (i=0; i<=number; i++)
    sortdata[i+1] = data[i];
  

  sort((unsigned long)number+1, sortdata);

  numberhalf = (int)((float)number/2.0);

  if (numberhalf % 2 != 0)
    med = ((sortdata[numberhalf] + sortdata[numberhalf+1])/2.0);
  else
    med = (sortdata[numberhalf+1]);
  
  free_vector(sortdata, 1, number+1);
  
  return med;
}


//takes 25% to both sides of the median value and calculates for this the mean and the variance

void median50mean(int number, float *data, double *mean, double *variance)
{
  float *sortdata;
  double med, value;
  double sum, sumsqr, numberhalfd;
  int i, numberhalf;

  sortdata = vector(1, number+1);
 
  for (i=0; i<=number; i++)
    sortdata[i+1] = data[i];
  
  sort((unsigned long)number+1, sortdata);

  //for (i=1; i<=number+1; i++)
  //printf("%i=%f\n", i, sortdata[i]);

  numberhalf = (int)((float)number/2.0);

  if (numberhalf % 2 != 0)
    med = ((sortdata[numberhalf] + sortdata[numberhalf+1])/2.0);
  else
    med = (sortdata[numberhalf+1]);

  sum = 0.0;
  sumsqr = 0.0;
  numberhalfd = 0.0;
  for (i=0; i<=number/2; i++)
    {
	value = sortdata[numberhalf/2+i+1];
	sum += (value/10000.0);
	sumsqr += (value/10000.0)*(value/10000.0);
	numberhalfd++;
    }

  free_vector(sortdata, 1, number+1);

  if ((numberhalfd-1.0 > 0.0) && (sumsqr-sum*sum/numberhalfd) > 0.0)
      (*variance) = sqrt((sumsqr-sum*sum/numberhalfd)/(numberhalfd-1.0))*10000.0;
  else
      (*variance) = -0.5;

  if ((double)number/2.0 > 0.0)
      (*mean) = sum/numberhalfd*10000.0;
  else
      (*mean) = -0.5;
}

int find_peak(double *scan, int delta)
{
  int i,j;
  double s, smax;

  i = 0;
  smax = 0.;
  for(j=8; j<248; j++)
    {
      s = scan[j-2*delta]-scan[j-delta]+scan[j]-scan[j+delta]+scan[j+2*delta];
      if(sqrt(s*s)>smax)
	{
	  smax = abs(s);
	  i = j;
	}
    }
  return(i);
}


int find_max(double *scan, int from, int to)
{
  int i,j;
  double s, smax;

  i = 0;
  smax = 0.;
  for(j=from; j<to; j++)
  {
      if (scan[j] > smax)
      {
	  smax = scan[j];
	  i = j;
      }
  }
  return(i);
}


void set_data(double *d, int *nn, int i, double dr, double di)
{
  int ii = idx(nn,i,-1,-1);
  d[ii] = dr;
  d[ii+1] = di;
}

void get_data(double *d, int *nn, int i, double* dr, double *di)
{
  int ii = idx(nn,i,-1,-1);
  *dr = d[ii];
  *di = d[ii+1];
}


void load_array(double *scan, double *data, int *nn, int direction)
{
  int i,k;

  for(i=0; i<256; i++)
    {
      if(direction == 1)
	k = i;
      else
	k = 255-i;
      set_data(data, nn, i, scan[k], 0.0);
    }
}
     

void ap(double *data, int *nn, double *amp, double *ph)
{
  int i, k;
  double dumr,dumi;
  
  for(i=nn[0]/2; i<nn[0]; i++)
    {
      k = i-nn[0]/2;
      
      get_data(data,nn,i,&dumr, &dumi);
      amp[k] = sqrt(dumr*dumr+dumi*dumi);
      if(dumr != 0.0)
	ph[k] = atan2(dumi,dumr);
      else
	ph[k] = 0.0;
    }
}
  
  

int doswap(int value)
{
  M4Byte swap;

  swap.i4d = value;
  swap4(&swap);
  value = swap.i4d;

  return value;
}


void copy_scan(FILE *from, FILE *to)
{
  int i, j, dum;

  for(i=0; i<nPtsScan; i++)
    for (j=0; j<6; j++)
      {
	fread(&dum, sizeof(int), 1, from);
	fwrite(&dum, sizeof(int), 1, to);
      }
}


void read_scan(FILE *inf, double *scan0, double *scan1, double *scan2, double **matrix4, double *matrixavg4, int *ch_baseline, fftw_plan fftw_f, fftw_plan fftw_b)
{
  int i, j, dum;
  //double sum[6];
  double *diff[6];
  double mean0, mean1, mean2;
  float plotx[nPtsScan], ploty[nPtsScan], ploty2[nPtsScan], ploty3[nPtsScan], ymin, ymax, ymin2, ymax2, ymin3, ymax3;

  double intleft, intright;
  double **matrix4_s;


  matrix4_s = dmatrix(0, 5, 0, nPtsScan-1);

  for (j=0; j<6; j++)
      diff[j] = dvector(0,nPtsScan-1);
  
  for(i=0; i<nPtsScan; i++)
      for (j=0; j<6; j++)
      {
	  fread(&dum, sizeof(int), 1, inf);
	  if (SWAP_BYTES == 1)
	      diff[j][i] = doswap(dum);
	  else
	      diff[j][i] = dum;
      }

  //take the derivative
  for(i=0; i<nPtsScan-1; i++)
    for (j=0; j<6; j++)
	diff[j][i] = diff[j][i+1]-diff[j][i];

  for (j=0; j<6; j++)
      diff[j][nPtsScan-1] = diff[j][nPtsScan-2];

  if (dobackgroundsub > 0)
  {
      for(i=0; i<nPtsScan; i++)
      {
	  matrix4[0][i] = 0.0;
	  matrix4[1][i] = 0.0;
	  matrix4[2][i] = 0.0;
	  matrix4[3][i] = 0.0;
	  matrix4[4][i] = 0.0;
	  matrix4[5][i] = 0.0;
      }

      for (j=0; j<6; j++)
	  smooth_signal(matrix4[j], nPtsScan, bgsubsmoothfactor, fftw_f, fftw_b, matrix4_s[j]);

      //subtracting the zero-matrix
      for (j=0; j<6; j++)
      {
	  ymin = 1e10;
	  ymax = -1e10;
	  ymin2 = 1e10;
	  ymax2 = -1e10;
	  ymin3 = 1e10;
	  ymax3 = -1e10;
	  
	  for(i=0; i<nPtsScan; i++)
	  {
	      if (doplotbg == 1)
	      {
		  plotx[i] = i;
		  ploty[i] = diff[j][i];
	      }
	      
	      if (dobackgroundsub == 1)
	      {
		  if (doplotbg == 1) ploty2[i] = matrix4_s[j][i];
		  diff[j][i] -= matrix4_s[j][i];
	      }
	      else
	      {
		  if (doplotbg == 1) ploty2[i] = matrixavg4[j];
		  diff[j][i] -= matrixavg4[j];
	      }

	      if (doplotbg == 1) ploty3[i] = diff[j][i];

	      if (ploty[i] > ymax) ymax = ploty[i];
	      if (ploty[i] < ymin) ymin = ploty[i];
	      if (ploty2[i] > ymax2) ymax2 = ploty2[i];
	      if (ploty2[i] < ymin2) ymin2 = ploty2[i];
	      if (ploty3[i] > ymax3) ymax3 = ploty3[i];
	      if (ploty3[i] < ymin3) ymin3 = ploty3[i];
	  }

	  if (doplotbg == 1)
	  {
	      cpgslct(cpg_windowbg);
	      cpgsci(1);
	      cpgenv(-1.0, nPtsScan, ymin2, ymax2, 0, 1);
	      cpglab(NameScan, "Intensity Background", "OPD");
	      cpgsci(2); //red
	      cpgline(nPtsScan-1, plotx, ploty2);

	      cpgslct(cpg_windowbg);
	      cpgsci(1);
	      cpgenv(-1.0, nPtsScan, ymin, ymax, 0, 1);
	      cpglab("", "Intensity Signal", "OPD");
	      cpgsci(1); //white
	      cpgline(nPtsScan-1, plotx, ploty);
	      cpgsci(2); //red
	      cpgline(nPtsScan-1, plotx, ploty2);

	      cpgsci(1);
	      cpgenv(-1.0, nPtsScan, ymin3, ymax3, 0, 1);
	      cpglab("", "Intensity Signal - Background", "OPD");
	      cpgsci(3);
	      cpgline(nPtsScan-1, plotx, ploty3);
	      cpgask(doplotwait);
	  }
      }
  }
  
  for(i=0; i<nPtsScan; i++)
    {
	scan0[i] = 0.0;
	scan1[i] = 0.0;
	scan2[i] = 0.0;
    }

  for(i=0; i<nPtsScan; i++)
    {
	//BA or AB
	if ((((ch_baseline[0] == 0) && (ch_baseline[1] == 1)) || ((ch_baseline[1] == 0) && (ch_baseline[0] == 1))) && (diff[0][i]+diff[1][i] != 0.0)) scan0[i] = (diff[0][i] - diff[1][i])/(diff[0][i] + diff[1][i]);
	if ((((ch_baseline[2] == 0) && (ch_baseline[3] == 1)) || ((ch_baseline[3] == 0) && (ch_baseline[2] == 1))) && (diff[2][i]+diff[3][i] != 0.0)) scan0[i] = (diff[2][i] - diff[3][i])/(diff[2][i] + diff[3][i]);
	if ((((ch_baseline[4] == 0) && (ch_baseline[5] == 1)) || ((ch_baseline[5] == 0) && (ch_baseline[4] == 1))) && (diff[4][i]+diff[5][i] != 0.0)) scan0[i] = (diff[4][i] - diff[5][i])/(diff[4][i] + diff[5][i]);
      
	//AC
	if ((ch_baseline[0] == 0) && (ch_baseline[1] == 2) && (diff[0][i]+diff[1][i] != 0.0)) scan1[i] = (diff[0][i] - diff[1][i])/(diff[0][i] + diff[1][i]);
	if ((ch_baseline[2] == 0) && (ch_baseline[3] == 2) && (diff[2][i]+diff[3][i] != 0.0)) scan1[i] = (diff[2][i] - diff[3][i])/(diff[2][i] + diff[3][i]);
	if ((ch_baseline[4] == 0) && (ch_baseline[5] == 2) && (diff[4][i]+diff[5][i] != 0.0)) scan1[i] = (diff[4][i] - diff[5][i])/(diff[4][i] + diff[5][i]);
      
	//BC
	if ((ch_baseline[0] == 1) && (ch_baseline[1] == 2) && (diff[0][i]+diff[1][i] != 0.0)) scan2[i] = (diff[0][i] - diff[1][i])/(diff[0][i] + diff[1][i]);
	if ((ch_baseline[2] == 1) && (ch_baseline[3] == 2) && (diff[2][i]+diff[3][i] != 0.0)) scan2[i] = (diff[2][i] - diff[3][i])/(diff[2][i] + diff[3][i]);
	if ((ch_baseline[4] == 1) && (ch_baseline[5] == 2) && (diff[4][i]+diff[5][i] != 0.0)) scan2[i] = (diff[4][i] - diff[5][i])/(diff[4][i] + diff[5][i]);
    }


  //subtracting the mean
  mean0 = 0;
  mean1 = 0;
  mean2 = 0;
  for (i=0; i<nPtsScan; i++)
    {
      mean0 += scan0[i];
      mean1 += scan1[i];
      mean2 += scan2[i];
    }
  mean0 /= nPtsScan;
  mean1 /= nPtsScan;
  mean2 /= nPtsScan;


  for(i=0; i<nPtsScan; i++)
    {
      scan0[i] -= mean0;
      scan1[i] -= mean1;
      scan2[i] -= mean2;
    }

  for (j=0; j<6; j++)
    free_dvector(diff[j],0,nPtsScan-1);
  
  free_dmatrix(matrix4_s, 0, 5, 0, nPtsScan-1);
}


void read_scan2(FILE *inf, double *scan0, double *scan1, double *scan2, int *ch_baseline)
{
  int i,dum0,dum1;
  double sum0, sum1, sum2;
  for(i=0; i<256; i++)
    {
      fread(&dum0, sizeof(int), 1, inf);
      if (SWAP_BYTES == 1) swap_bytes(&dum0);

      fread(&dum1, sizeof(int), 1, inf);
      if (SWAP_BYTES == 1) swap_bytes(&dum1);

      //scan0[i] = dum0-dum1;
      //AB or BA
      if (((ch_baseline[0] == 0) && (ch_baseline[1] == 1)) || ((ch_baseline[1] == 0) && (ch_baseline[0] == 1))) scan0[i] = dum0-dum1;
      if (((ch_baseline[2] == 0) && (ch_baseline[3] == 1)) || ((ch_baseline[3] == 0) && (ch_baseline[2] == 1))) scan0[i] = dum0-dum1;
      if (((ch_baseline[4] == 0) && (ch_baseline[5] == 1)) || ((ch_baseline[5] == 0) && (ch_baseline[4] == 1))) scan0[i] = dum0-dum1;


      fread(&dum0, sizeof(int), 1, inf);
      if (SWAP_BYTES == 1) swap_bytes(&dum0);

      fread(&dum1, sizeof(int), 1, inf);
      if (SWAP_BYTES == 1) swap_bytes(&dum1);

      //scan1[i] = dum0-dum1;
      //AC
      if ((ch_baseline[0] == 0) && (ch_baseline[1] == 2)) scan1[i] = dum0-dum1;
      if ((ch_baseline[2] == 0) && (ch_baseline[3] == 2)) scan1[i] = dum0-dum1;
      if ((ch_baseline[4] == 0) && (ch_baseline[5] == 2)) scan1[i] = dum0-dum1;


      fread(&dum0, sizeof(int), 1, inf);
      if (SWAP_BYTES == 1) swap_bytes(&dum0);

      fread(&dum1, sizeof(int), 1, inf);
      if (SWAP_BYTES == 1) swap_bytes(&dum1);

      //scan2[i] = dum0-dum1;
      //AC
      if ((ch_baseline[0] == 1) && (ch_baseline[1] == 2)) scan2[i] = dum0-dum1;
      if ((ch_baseline[2] == 1) && (ch_baseline[3] == 2)) scan2[i] = dum0-dum1;
      if ((ch_baseline[4] == 1) && (ch_baseline[5] == 2)) scan2[i] = dum0-dum1;
    }
  sum0 = 0;
  sum1 = 0;
  sum2 = 0;

  for(i=0; i<255; i++)
    {
      scan0[i] = scan0[i+1]-scan0[i];
      scan1[i] = scan1[i+1]-scan1[i];
      scan2[i] = scan2[i+1]-scan2[i];

      sum0 += scan0[i];
      sum1 += scan1[i];
      sum2 += scan2[i];

    }
  scan0[255] = scan0[254];
  scan1[255] = scan1[254];
  scan2[255] = scan2[254];
  sum0 += scan0[255];
  sum1 += scan1[255];
  sum2 += scan2[255];
  sum0 /= 256;
  sum1 /= 256;
  sum2 /= 256;

  for(i=0; i<256; i++)
    {
      scan0[i] -= sum0;
      scan1[i] -= sum1;
      scan2[i] -= sum2;
    }
}



void read_singlescan(FILE *inf, double *noise)
{
    int j, dum;
    
    for (j=0; j<6; j++)
    {
	fread(&dum, sizeof(int), 1, inf);
	if (SWAP_BYTES == 1)
	    noise[j] = doswap(dum);
	else
	    noise[j] = dum;
    }
}


void read_scan_deriv(FILE *inf, double *deriv0, double *deriv1, double *deriv2, double *deriv3, double *deriv4, double *deriv5)
{
  int i, j, dum;
  double *diff[6];

  for (j=0; j<6; j++)
    diff[j] = dvector(0,nPtsScan-1);

  for(i=0; i<nPtsScan; i++)
    for (j=0; j<6; j++)
      {
	fread(&dum, sizeof(int), 1, inf);
	if (SWAP_BYTES == 1)
	    diff[j][i] = doswap(dum);
	else
	    diff[j][i] = dum;
      }
  

  //get the derivative
  for(i=0; i<nPtsScan-1; i++)
    for (j=0; j<6; j++)
	diff[j][i] = diff[j][i+1]-diff[j][i];
  
  for (j=0; j<6; j++)
      diff[j][nPtsScan-1] = diff[j][nPtsScan-2];

  for(i=0; i<nPtsScan; i++)
    {
      deriv0[i] = diff[0][i];
      deriv1[i] = diff[1][i];
      deriv2[i] = diff[2][i];
      deriv3[i] = diff[3][i];
      deriv4[i] = diff[4][i];
      deriv5[i] = diff[5][i];
    }
 
  for (j=0; j<6; j++)
    free_dvector(diff[j],0,nPtsScan-1);
}


int read_scan_photcorr(FILE *inf, double *scan0, double *scan1, double *scan2, double *matrix0level, double *matrix1level, double *matrix2level, double **matrix4, double *matrixavg4, int *ch_baseline, fftw_plan fftw_f, fftw_plan fftw_b)
{

    int i, j, k, l;
    double mean;
    int count;
    float **corr;
    float **pixel, **pixel_orig;
    float **pixel_smooth;
    double intensity[3];
    double d0, d1, d2;
    double M00, M10, M20, M01, M11, M21, M02, M12, M22;
    double uncorr[3];
    float **intensity_, **intensity_s;

    double norm_pixel[6], norm_pixel_smooth[6];
    double norm_gauss;

    double kappa0[6];
    double kappa1[6];
    double kappa2[6];
    double check[3];

    int dum;
    double *diff[6], *diff_orig[6];
    float plotx[nPtsScan], ploty[nPtsScan], ploty2[nPtsScan], ploty3[nPtsScan], ymin, ymax, ymin2, ymax2, ymin3, ymax3;

    double pixel0[nPtsScan], pixel1[nPtsScan], pixel2[nPtsScan], pixel3[nPtsScan], pixel4[nPtsScan], pixel5[nPtsScan];

    fftw_complex scan[3*nPtsScan], gauss[3*nPtsScan], scan_smooth[3*nPtsScan];
    fftw_complex ftscan[3*nPtsScan], ftgauss[3*nPtsScan];
    
    float **mat1, **mat2, **mat3, *col;
    float d;
    int *indx;

    int indexpixel;
    double **matrix4_s;
    int rejectscan;

    
    matrix4_s = dmatrix(0, 5, 0, nPtsScan-1);

    pixel=matrix(0,5,0,nPtsScan-1);
    pixel_smooth=matrix(0,5,0,nPtsScan-1);
    pixel_orig=matrix(0,5,0,nPtsScan-1);
    corr=matrix(0,2,0,nPtsScan-1);
    intensity_=matrix(0,2,0,nPtsScan-1);
    intensity_s=matrix(0,2,0,nPtsScan-1);

    mat1=matrix(1,3,1,3);
    mat2=matrix(1,3,1,3);
    mat3=matrix(1,3,1,3);
    col=vector(1,3);
    indx=ivector(1,3);

    
    for (j=0; j<6; j++)
    {
	diff[j] = dvector(0,nPtsScan-1);
	diff_orig[j] = dvector(0,nPtsScan-1);
    }
    
    for(i=0; i<nPtsScan; i++)
	for (j=0; j<6; j++)
	{
	    fread(&dum, sizeof(int), 1, inf);
	    if (SWAP_BYTES == 1)
		diff[j][i] = doswap(dum);
	    else
		diff[j][i] = dum;
	    diff_orig[j][i] = diff[j][i];
	}


    //take the derivative
    for(i=0; i<nPtsScan-1; i++)
	for (j=0; j<6; j++)
	    diff[j][i] = diff[j][i+1]-diff[j][i];
    for (j=0; j<6; j++)
	diff[j][nPtsScan-1] = diff[j][nPtsScan-2];


    for(i=0; i<nPtsScan-1; i++)
	for (j=0; j<6; j++)
	    diff_orig[j][i] = diff_orig[j][i+1]-diff_orig[j][i];
    for (j=0; j<6; j++)
	diff_orig[j][nPtsScan-1] = diff_orig[j][nPtsScan-2];
    
    
    if (dobackgroundsub > 0)
    {
	for (j=0; j<6; j++)
	    smooth_signal(matrix4[j], nPtsScan, bgsubsmoothfactor, fftw_f, fftw_b, matrix4_s[j]);
	
	//subtracting the zero-matrix
	for (j=0; j<6; j++)
	{
	    ymin = 1e10;
	    ymax = -1e10;
	    ymin2 = 1e10;
	    ymax2 = -1e10;
	    ymin3 = 1e10;
	    ymax3 = -1e10;
	    
	    for(i=0; i<nPtsScan; i++)
	    {
		if (doplotbg == 1)
		{
		    plotx[i] = i;
		    ploty[i] = diff[j][i];
		}
		
		if (dobackgroundsub == 1)
		{
		    if (doplotbg == 1) ploty2[i] = matrix4_s[j][i];
		    diff[j][i] -= matrix4_s[j][i];
		}
		else
		{
		    if (doplotbg == 1) ploty2[i] = matrixavg4[j];
		    diff[j][i] -= matrixavg4[j];
		}
		
		if (doplotbg == 1) ploty3[i] = diff[j][i];
		
		if (ploty[i] > ymax) ymax = ploty[i];
		if (ploty[i] < ymin) ymin = ploty[i];
		if (ploty2[i] > ymax2) ymax2 = ploty2[i];
		if (ploty2[i] < ymin2) ymin2 = ploty2[i];
		if (ploty3[i] > ymax3) ymax3 = ploty3[i];
		if (ploty3[i] < ymin3) ymin3 = ploty3[i];
	    }
	    
	    
	    
	    if (doplotbg == 1)
	    {
		cpgslct(cpg_windowbg);
		cpgsci(1);
		cpgenv(-1.0, nPtsScan, ymin2, ymax2, 0, 1);
		cpglab("OPD", "Intensity Background", NameScan);
		cpgsci(2); //red
		cpgline(nPtsScan-1, plotx, ploty2);
		
		cpgslct(cpg_windowbg);
		cpgsci(1);
		cpgenv(-1.0, nPtsScan, ymin, ymax, 0, 1);
		cpglab("OPD", "Intensity Signal", "");
		cpgsci(1); //white
		cpgline(nPtsScan-1, plotx, ploty);
		cpgsci(2); //red
		cpgline(nPtsScan-1, plotx, ploty2);
		
		cpgsci(1);
		cpgenv(-1.0, nPtsScan, ymin3, ymax3, 0, 1);
		cpglab("OPD", "Intensity Signal - Background", "");
		cpgsci(3); //green
		cpgline(nPtsScan-1, plotx, ploty3);
		cpgask(doplotwait);
	    }
	}
    }

    for (i=0; i<nPtsScan; i++)
	for (j=0; j<6; j++)
	{
	    pixel[j][i] = diff[j][i];
	    pixel_orig[j][i] = diff[j][i];
	}
        

    //*** Smoothing
    for (j=0; j<6; j++)
	smooth_signal_float(pixel[j], nPtsScan, photcorrsmoothfactor1, fftw_f, fftw_b, pixel_smooth[j]);

    //*** Determine Kappa matrix (including bias)!
    M00 = matrix0level[0] / sqrt(matrix1level[0] * matrix2level[0]) + 
	matrix0level[1] / sqrt(matrix1level[1] * matrix2level[1]);
    M11 = matrix1level[2] / sqrt(matrix0level[2] * matrix2level[2]) + 
	matrix1level[3] / sqrt(matrix0level[3] * matrix2level[3]);
    M22 = matrix2level[4] / sqrt(matrix0level[4] * matrix1level[4]) + 
	matrix2level[5] / sqrt(matrix0level[5] * matrix1level[5]);
    
    
    kappa0[2] = matrix0level[2];// / matrix2level[2];
    kappa0[3] = matrix0level[3];// / matrix2level[3];
    kappa0[4] = matrix0level[4];// / matrix1level[4];
    kappa0[5] = matrix0level[5];// / matrix1level[5];
    
    kappa1[0] = matrix1level[0];// / matrix2level[0];
    kappa1[1] = matrix1level[1];// / matrix2level[1];
    kappa1[4] = matrix1level[4];// / matrix0level[4];
    kappa1[5] = matrix1level[5];// / matrix0level[5];
    
    kappa2[0] = matrix2level[0];// / matrix1level[0];
    kappa2[1] = matrix2level[1];// / matrix1level[1];
    kappa2[2] = matrix2level[2];// / matrix0level[2];
    kappa2[3] = matrix2level[3];// / matrix0level[3];
    
    
    M10 = sqrt(matrix1level[0] / matrix2level[0]) + sqrt(matrix1level[1] / matrix2level[1]);
    M20 = sqrt(matrix2level[0] / matrix1level[0]) + sqrt(matrix2level[1] / matrix1level[1]);
    
    M01 = sqrt(matrix0level[2] / matrix2level[2]) + sqrt(matrix0level[3] / matrix2level[3]);
    M21 = sqrt(matrix2level[2] / matrix0level[2]) + sqrt(matrix2level[3] / matrix0level[3]);
    
    M02 = sqrt(matrix0level[4] / matrix1level[4]) + sqrt(matrix0level[5] / matrix1level[5]);
    M12 = sqrt(matrix1level[4] / matrix0level[4]) + sqrt(matrix1level[5] / matrix0level[5]);
    
    //invert matrix
    mat1[1][1]=M00;
    mat1[1][2]=M01;
    mat1[1][3]=M02;
    mat1[2][1]=M10;
    mat1[2][2]=M11;
    mat1[2][3]=M12;
    mat1[3][1]=M20;
    mat1[3][2]=M21;
    mat1[3][3]=M22;

    for (j=1; j<=3; j++)
	for (k=1; k<=3; k++)
	    mat3[k][j] = mat1[k][j];
    
    //inverse matrix
    ludcmp(mat1, 3, indx, &d);
    for (j=1; j<=3; j++)
    {
	for (k=1; k<=3; k++)
		col[k] = 0.0;
	col[j]=1.0;
	lubksb(mat1, 3, indx, col);
	for (k=1; k<=3; k++)
	    mat2[k][j] = col[k];
    }

   //test inverse matrix
   for (j=1; j<=3; j++)
   for (k=1; k<=3; k++)
   mat1[k][j] = mat3[k][j];
   /*
   printf("%f %f %f\n", mat2[1][1], mat2[1][2], mat2[1][3]);
   printf("%f %f %f\n", mat2[2][1], mat2[2][2], mat2[2][3]);
   printf("%f %f %f\n", mat2[3][1], mat2[3][2], mat2[3][3]);
   */   
   for (k=1; k<=3; k++)
       for (j=1; j<=3; j++)
       {
	   mat3[k][j] = 0.0;
     
	   for (l=1; l<=3; l++)
	       mat3[k][j] += (mat1[k][l] * mat2[l][j]);
       }
   /*     
     printf("%f %f %f\n", mat3[1][1], mat3[1][2], mat3[1][3]);
     printf("%f %f %f\n", mat3[2][1], mat3[2][2], mat3[2][3]);
     printf("%f %f %f\n", mat3[3][1], mat3[3][2], mat3[3][3]);
   */    
    for(i=0; i<nPtsScan; i++)
    {
	scan0[i] = 0.0;
	scan1[i] = 0.0;
	scan2[i] = 0.0;

	d0 = pixel_smooth[0][i] / sqrt(matrix1level[0] * matrix2level[0]) + 
             pixel_smooth[1][i] / sqrt(matrix1level[1] * matrix2level[1]);
	d1 = pixel_smooth[2][i] / sqrt(matrix0level[2] * matrix2level[2]) + 
	     pixel_smooth[3][i] / sqrt(matrix0level[3] * matrix2level[3]);
	d2 = pixel_smooth[4][i] / sqrt(matrix0level[4] * matrix1level[4]) + 
	     pixel_smooth[5][i] / sqrt(matrix0level[5] * matrix1level[5]);
	
	intensity[0] = d0*mat2[1][1] + d1*mat2[2][1] + d2*mat2[3][1];
	intensity[1] = d0*mat2[1][2] + d1*mat2[2][2] + d2*mat2[3][2];
	intensity[2] = d0*mat2[1][3] + d1*mat2[2][3] + d2*mat2[3][3];
	
	intensity_[0][i] = intensity[0];
	intensity_[1][i] = intensity[1];
	intensity_[2][i] = intensity[2];
    }
        

    //*** smooth photometric signal
    for (j=0; j<3; j++)
	smooth_signal_float(intensity_[j], nPtsScan, photcorrsmoothfactor2, fftw_f, fftw_b, intensity_s[j]);

    for(i=0; i<nPtsScan; i++)
    {
	for (j=0; j<=2; j++)
	{
	    //intensity[j] = -intensity[j];

	    if (intensity_[j][i] > 0.0)
	    {
		for (k=0; k<3; k++)
		    intensity_[k][i] = 0.0;
	    }
	}


	//correct interferogram
	if ((intensity_[1][i]*intensity_[2][i] != 0.0) && (intensity_s[1][i]*intensity_s[2][i] != 0.0))
	{
	    pixel[0][i] = (pixel[0][i] - kappa1[0]*intensity_[1][i] - kappa2[0]*intensity_[2][i]) / 
                                    (2.0*sqrt(kappa1[0]*intensity_s[1][i]*kappa2[0]*intensity_s[2][i]));
	    pixel[1][i] = (pixel[1][i] - kappa1[1]*intensity_[1][i] - kappa2[1]*intensity_[2][i]) / 
                                    (2.0*sqrt(kappa1[1]*intensity_s[1][i]*kappa2[1]*intensity_s[2][i]));
	}
	else
	{
	    pixel[0][i] = -9999.9;
	    pixel[1][i] = -9999.9;
	}
	
	if ((intensity_[0][i]*intensity_[2][i] != 0.0) && (intensity_s[0][i]*intensity_s[2][i] != 0.0))
	{
	    pixel[2][i] = (pixel[2][i] - kappa0[2]*intensity_[0][i] - kappa2[2]*intensity_[2][i]) /
                                    (2.0*sqrt(kappa0[2]*intensity_s[0][i]*kappa2[2]*intensity_s[2][i]));
	    pixel[3][i] = (pixel[3][i] - kappa0[3]*intensity_[0][i] - kappa2[3]*intensity_[2][i]) / 
                                    (2.0*sqrt(kappa0[3]*intensity_s[0][i]*kappa2[3]*intensity_s[2][i]));
	}
	else
	{
	    pixel[2][i] = -9999.9;
	    pixel[3][i] = -9999.9;
	}
	
	if ((intensity_[0][i]*intensity_[1][i] != 0.0) && (intensity_s[0][i]*intensity_s[1][i] != 0.0))
	{
	    pixel[4][i] = (pixel[4][i] - kappa0[4]*intensity_[0][i] - kappa1[4]*intensity_[1][i]) /
                                    (2.0*sqrt(kappa0[4]*intensity_s[0][i]*kappa1[4]*intensity_s[1][i]));
	    pixel[5][i] = (pixel[5][i] - kappa0[5]*intensity_[0][i] - kappa1[5]*intensity_[1][i]) /
                                    (2.0*sqrt(kappa0[5]*intensity_s[0][i]*kappa1[5]*intensity_s[1][i]));
	}
	else
	{
	    pixel[4][i] = -9999.9;
	    pixel[5][i] = -9999.9;
	}
    }


    for (j=0; j<=2; j++)
    {
	rejectscan = 0;
	for (i=0; i<nPtsScan; i++)
	    if ((pixel[2*j][i] < -9999.0) || (pixel[2*j+1][i] < -9999.0))
		rejectscan = 1;

	if (rejectscan == 0)
	    for (i=0; i<nPtsScan; i++)
		corr[j][i] = (pixel[2*j][i] - pixel[2*j+1][i])/2.0;
	else
	{
	    for (i=0; i<nPtsScan; i++)
		corr[j][i] = 0.0;
	}
    }

    //if (rejectscan == 1) printf("Scan rejected!\n");

    //subtracting the mean
    for (j=0; j<=2; j++)
    {
	mean = 0;
	count = 0;
	
	for (i=0; i<nPtsScan; i++)
	    if (corr[j][i] > -99.0)
	    {
		mean += corr[j][i];
		count++;
	    }
	mean /= (float)count;
	
	for (i=0; i<nPtsScan; i++)
	    if (corr[j][i] > -99.0) 
		corr[j][i] -= mean;
    }


    if (doplot0 == 1)
    {
	cpgslct(cpg_window0);

	cpgsubp(1, 1);
	cpgeras;
	cpgsubp(2, 3);

	cpgask(doplotwait);
	
	for (j=0; j<2; j++)
	{
	    //plot data
	    ymin = 1e10;
	    ymax = 1e-10;
	    for (i=0; i<nPtsScan; i++)
	    {
		plotx[i] = i;
		ploty[i] = pixel_orig[j][i];

		if (j == 0) ploty2[i] = -kappa1[0]*intensity_[1][i] - kappa2[0]*intensity_[2][i];
		if (j == 1) ploty2[i] = -kappa1[1]*intensity_[1][i] - kappa2[1]*intensity_[2][i];

		if (j == 0) ploty3[i] = -kappa1[0]*intensity_s[1][i] - kappa2[0]*intensity_s[2][i];
		if (j == 1) ploty3[i] = -kappa1[1]*intensity_s[1][i] - kappa2[1]*intensity_s[2][i];

		if (j == 2) ploty2[i] = -kappa0[2]*intensity_[0][i] - kappa2[2]*intensity_[2][i];
		if (j == 3) ploty2[i] = -kappa0[3]*intensity_[0][i] - kappa2[3]*intensity_[2][i];

		if (j == 2) ploty3[i] = -kappa0[2]*intensity_s[0][i] - kappa2[2]*intensity_s[2][i];
		if (j == 3) ploty3[i] = -kappa0[3]*intensity_s[0][i] - kappa2[3]*intensity_s[2][i];

		if (j == 4) ploty2[i] = -kappa0[4]*intensity_[0][i] - kappa1[4]*intensity_[1][i];
		if (j == 5) ploty2[i] = -kappa0[5]*intensity_[0][i] - kappa1[5]*intensity_[1][i];

		if (j == 4) ploty3[i] = -kappa0[4]*intensity_s[0][i] - kappa1[4]*intensity_s[1][i];
		if (j == 5) ploty3[i] = -kappa0[5]*intensity_s[0][i] - kappa1[5]*intensity_s[1][i];

		if (ploty[i] > ymax) ymax = ploty[i];
		if (ploty[i] < ymin) ymin = ploty[i];
		if (ploty2[i] > ymax) ymax = ploty2[i];
		if (ploty2[i] < ymin) ymin = ploty2[i];
		if (ploty3[i] > ymax) ymax = ploty3[i];
		if (ploty3[i] < ymin) ymin = ploty3[i];
	    }
	
	    cpgsci(1);
	    cpgenv(-1.0, nPtsScan, ymin, ymax, 0, 1);
	    if (j == 0) cpglab("Raw Signal 0", "Intensity", NameScan);
	    if (j == 1) cpglab("Raw Signal 1", "Intensity", NameScan);
	    cpgsci(3+j);
	    cpgline(nPtsScan-1, plotx, ploty);
	    cpgsci(2);
	    cpgline(nPtsScan-1, plotx, ploty2);
	    cpgsci(3);
	    cpgline(nPtsScan-1, plotx, ploty3);
	}

	for (j=0; j<2; j++)
	{
	    //plot data
	    ymin = 1e10;
	    ymax = 1e-10;
	    for (i=0; i<nPtsScan; i++)
	    {
		plotx[i] = i;
		ploty[i] = corr[j][i];
		
		if (ploty[i] > ymax) ymax = ploty[i];
		if (ploty[i] < ymin) ymin = ploty[i];
	    }
	
	    cpgsci(1);
	    cpgenv(-1.0, nPtsScan, ymin, ymax, 0, 1);
	    if (j == 0) cpglab("", "Intensity", "Corrected Signal 0");
	    if (j == 1) cpglab("", "Intensity", "Corrected Signal 1");
	    cpgsci(3+j);
	    cpgline(nPtsScan-1, plotx, ploty);
	}

	//plot intensities
	ymin = 1e10;
	ymax = 1e-10;
	for (i=0; i<nPtsScan; i++)
	{
	    plotx[i] = i;
	    ploty[i] = intensity_[1][i];
	    ploty2[i] = intensity_[2][i];
	    
	    if (ploty[i] > ymax) ymax = ploty[i];
	    if (ploty[i] < ymin) ymin = ploty[i];
	    if (ploty2[i] > ymax) ymax = ploty2[i];
	    if (ploty2[i] < ymin) ymin = ploty2[i];
	}
	
	cpgsci(1);
	cpgenv(-1.0, nPtsScan, ymin, ymax, 0, 1);
	cpglab("", "Intensity", "Photometry");
	cpgsci(3);
	cpgline(nPtsScan-1, plotx, ploty);
	cpgsci(4);
	cpgline(nPtsScan-1, plotx, ploty2);



	//plot corr
	ymin = 1e10;
	ymax = 1e-10;
	for (i=0; i<nPtsScan; i++)
	{
	    plotx[i] = i;
	    ploty[i] = corr[0][i];
	    
	    if (ploty[i] > ymax) ymax = ploty[i];
	    if (ploty[i] < ymin) ymin = ploty[i];
	}
	
	cpgsci(1);
	cpgenv(-1.0, nPtsScan, ymin, ymax, 0, 1);
	cpglab("", "Intensity", "Corrected Interferogram = Difference between Corrected Signals");
	cpgsci(1);
	cpgline(nPtsScan-1, plotx, ploty);
    }

    for (i=0; i<nPtsScan; i++)
    {
	//BA or AB
	if (((ch_baseline[0] == 0) && (ch_baseline[1] == 1)) || ((ch_baseline[1] == 0) && (ch_baseline[0] == 1))) scan0[i] = corr[0][i];
	if (((ch_baseline[2] == 0) && (ch_baseline[3] == 1)) || ((ch_baseline[3] == 0) && (ch_baseline[2] == 1))) scan0[i] = corr[1][i];
	if (((ch_baseline[4] == 0) && (ch_baseline[5] == 1)) || ((ch_baseline[5] == 0) && (ch_baseline[4] == 1))) scan0[i] = corr[2][i];
	
	//AC
	if ((ch_baseline[0] == 0) && (ch_baseline[1] == 2) && (pixel[0][i]+pixel[1][i] != 0.0)) scan1[i] = corr[0][i];
	if ((ch_baseline[2] == 0) && (ch_baseline[3] == 2) && (pixel[2][i]+pixel[3][i] != 0.0)) scan1[i] = corr[1][i];
	if ((ch_baseline[4] == 0) && (ch_baseline[5] == 2) && (pixel[4][i]+pixel[5][i] != 0.0)) scan1[i] = corr[2][i];
	
	//BC
	if ((ch_baseline[0] == 1) && (ch_baseline[1] == 2) && (pixel[0][i]+pixel[1][i] != 0.0)) scan2[i] = corr[0][i];
	if ((ch_baseline[2] == 1) && (ch_baseline[3] == 2) && (pixel[2][i]+pixel[3][i] != 0.0)) scan2[i] = corr[1][i];
	if ((ch_baseline[4] == 1) && (ch_baseline[5] == 2) && (pixel[4][i]+pixel[5][i] != 0.0)) scan2[i] = corr[2][i];
    }


    free_matrix(mat1,1,3,1,3);
    free_matrix(mat2,1,3,1,3);
    free_matrix(mat3,1,3,1,3);
    free_vector(col,1,3);
    free_ivector(indx,1,3);

    free_matrix(pixel,0,5,0,nPtsScan-1);
    free_matrix(pixel_smooth,0,5,0,nPtsScan-1);
    free_matrix(pixel_orig,0,5,0,nPtsScan-1);
    free_matrix(corr,0,2,0,nPtsScan-1);
    free_matrix(intensity_, 0,2,0,nPtsScan-1);
    free_matrix(intensity_s, 0,2,0,nPtsScan-1);
    
    free_dmatrix(matrix4_s, 0, 5, 0, nPtsScan-1);

    for (j=0; j<6; j++)
    {
	free_dvector(diff[j],0,nPtsScan-1);
	free_dvector(diff_orig[j],0,nPtsScan-1);
    }

    if (rejectscan == 1)
	return 0;
    else
	return 1;
}


double ut2jd(int yy, int mm, int dd, int h, int m_, int s)
{
  double d, jd;
  int a, y, m, b, c, e;
  
  d = (double)dd + ((double)h + (double)m_ / 60.0 + (double)s / 3600.0) / 24.0;
  
  if (mm > 2)
    { 
      y = yy;
      m = mm;
    }
  else
    {
      y = yy - 1;
      m = mm + 12;
    }

  a = y / 100;
  b = 2 - a + (a / 4);
  c = (int)(30.6001 * ((double)m + 1.0));
  e = (int)(365.25 * ((double)y + 4716.0));

  jd = d + (double)b - 1524.5 + (double)c + (double)e;

  return jd;
}


//calib=0: real data; calib=1: calibration data (at least one shutter blocked)

void read_header(char* file, char* inidir, int *scans, int *fixeddelay, int *calib, int *objid, double *jd, double *ustart, double *uend, double *vstart, double *vend, int *ch_baseline, double *wavelength, double *tel0, double *tel1, double *tel2, double *fringe_freq, int scriptnrall, FILE *streaminfo)
{
  FILE *stream, *stream2;
  char text_[200], text1[5], text2[5];
  char text[10][100], filename[100], filename2[100];
  char oldid[30], objid_[30];
  int i, j, n, baselines[3];
  int dd, mm, yy, h, m, s, h2, m2, s2;
  int newid, shutter[3];
  double vs[3], ve[3], us[3], ue[3];
  double jd2;

  char sta_tel0[5], sta_tel1[5], sta_tel2[5];
  char sta_tel0_0[30], sta_tel0_1[30], sta_tel0_2[30];
  char sta_tel1_0[30], sta_tel1_1[30], sta_tel1_2[30];
  char sta_tel2_0[30], sta_tel2_1[30], sta_tel2_2[30];
  char arr_name[20];
  char tar_star[30], tar_ra[30], tar_dec[30];
  char ins_combiner[30], ins_wavelength[10], ins_name[20];
  
  char vis_date[10];
  double vis_mjd, vis_time, vis_int_time;

  int found, objidnew, count, count2, listcount, objidlistnr[max_objects];
  char objidlist[30][max_objects], c, c2;
  int clist[10], rlist[10];
  float rdlist[10], rdiameter;

  if (file_exist(file))
    {
      stream = fopen(file,"r");

      *calib = 0;
      *objid = 0;
      strcpy(objid_, "");

      i = 0;
      if (stream != NULL)
	{
	  while (feof(stream) == 0)
	    {
	      i++;
	  
	      //read whole line in text_
	      j = 0;
	      for (;;)
		{
		  text_[j]  = getc(stream);
		  if (text_[j] == 10) break;
		  if (feof(stream) != 0) break;
		  j++;
		}
	      
	      //separate words in text[0]
	      if (j > 10)
		{
		  sscanf(text_, "%s %s %s %s %s %s %s %s %s %s", text[0], text[1], text[2], text[3], text[4], text[5], text[6], text[7], text[8], text[9]);
		  
		  if (strcasecmp(text[0], "star") == 0)
		    strcpy(objid_, text[2]);
		    //*objid = atoi(text[2]);
		  
		  if (strcasecmp(text[0], "scans") == 0)
		    *scans = atoi(text[2]);

		  //target
		  if (strcasecmp(text[0], "star") == 0)
		    strcpy(tar_star, text[2]);

		  if (strcasecmp(text[0], "RA") == 0)
		    strcpy(tar_ra, text[3]);

		  if (strcasecmp(text[0], "DEC") == 0)
		    strcpy(tar_dec, text[3]);

		  //instrument
		  if (strcasecmp(text[0], "combiner") == 0)
		    strcpy(ins_combiner, text[2]);
		  
		  if ((strcasecmp(text[0], "telescope") == 0) && ((strcasecmp(text[1], "a") == 0)))
		    {
		      tel0[0] = atof(text[7]);
		      tel0[1] = atof(text[8]);
		      tel0[2] = atof(text[9]);

		      strcpy(sta_tel0, "     ");
		      strncpy(sta_tel0, text[6], 4);
		      strcpy(sta_tel0_0, text[7]);
		      strcpy(sta_tel0_1, text[8]);
		      strcpy(sta_tel0_2, text[9]);
		    }

		  if ((strcasecmp(text[0], "telescope") == 0) && ((strcasecmp(text[1], "b") == 0)))
		    {
		      tel1[0] = atof(text[7]);
		      tel1[1] = atof(text[8]);
		      tel1[2] = atof(text[9]);

		      strcpy(sta_tel1, "     ");
		      strncpy(sta_tel1, text[6], 4);
		      strcpy(sta_tel1_0, text[7]);
		      strcpy(sta_tel1_1, text[8]);
		      strcpy(sta_tel1_2, text[9]);
		    }

		  if ((strcasecmp(text[0], "telescope") == 0) && ((strcasecmp(text[1], "c") == 0)))
		    {
		      tel2[0] = atof(text[7]);
		      tel2[1] = atof(text[8]);
		      tel2[2] = atof(text[9]);

		      strcpy(sta_tel2, "     ");
		      strncpy(sta_tel2, text[6], 4);
		      strcpy(sta_tel2_0, text[7]);
		      strcpy(sta_tel2_1, text[8]);
		      strcpy(sta_tel2_2, text[9]);
		    }

		  if ((strcasecmp(text[0], "fixed") == 0) && (strcasecmp(text[1], "delay") == 0))
		    {
		      if (strcasecmp(text[3], "a") == 0) *fixeddelay = 0;
		      if (strcasecmp(text[3], "b") == 0) *fixeddelay = 1;
		      if (strcasecmp(text[3], "c") == 0) *fixeddelay = 2;
		    }
		  
		  if (strcasecmp(text[0], "shutters") == 0)
		    {
		      for (n=0; n<3; n++)
			if (strcasecmp(text[2+n], "blocked") == 0)
			  shutter[n] = 0;
		        else
			  shutter[n] = 1;
		      
		      if ((shutter[0] == 1) && (shutter[1] == 0) && (shutter[2] == 0))
			*calib = 1;

		      if ((shutter[0] == 0) && (shutter[1] == 1) && (shutter[2] == 0))
			*calib = 2;
		      
		      if ((shutter[0] == 0) && (shutter[1] == 0) && (shutter[2] == 1))
			*calib = 3;
		      
		      if ((shutter[0] == 0) && (shutter[1] == 0) && (shutter[2] == 0))
			*calib = 4;

		    }
		  
		  
		  if ((strcasecmp(text[0], "ut") == 0) && (strcasecmp(text[1], "date") == 0))
		    {
		      yy = atoi(text[3]);
		      mm = atoi(text[4]);
		      dd = atoi(text[5]);
		      
		      strcpy(vis_date, "");
		      strcat(vis_date, text[3]); strcat(vis_date, "-");
		      strcat(vis_date, text[4]); strcat(vis_date, "-");
		      strcat(vis_date, text[5]);
		    }
		  
		  if ((strcasecmp(text[0], "ut") == 0) && (strcasecmp(text[1], "time") == 0))
		    {
		      h = atoi(text[3]);
		      m = atoi(text[4]);
		      s = atoi(text[5]);

		      h2 = atoi(text[6]);
		      m2 = atoi(text[7]);
		      s2 = atoi(text[8]);
		      
		      *jd = ut2jd(yy, mm, dd, h, m, s);
		      jd2 = ut2jd(yy, mm, dd, h2, m2, s2);

		      vis_mjd = (*jd+jd2)/2.0-2400000.0;
		      vis_time = ((s + 60.0*m + 3600.0*h)+(s2 + 60.0*m2 + 3600.0*h2))/2.0;
		      vis_int_time = (s2-s) + (m2-m)*60.0 + (h2-h)*3600.0;
		    }
		  
		  if (strcasecmp(text[0], "uAB") == 0)
		    {
		      us[0] = atof(text[3]);
		      ue[0] = atof(text[4]);
		    }
		  
		  if (strcasecmp(text[0], "vAB") == 0)
		    {
		      vs[0] = atof(text[3]);
		      ve[0] = atof(text[4]);
		    }
		  
		  if (strcasecmp(text[0], "uAC") == 0)
		    {
		      us[1] = atof(text[3]);
		      ue[1] = atof(text[4]);
		    }

		  if (strcasecmp(text[0], "vAC") == 0)
		    {
		      vs[1] = atof(text[3]);
		      ve[1] = atof(text[4]);
		    }
		  
		  if (strcasecmp(text[0], "uBC") == 0)
		    {
		      us[2] = atof(text[3]);
		      ue[2] = atof(text[4]);
		    }
		  
		  if (strcasecmp(text[0], "vBC") == 0)
		    {
		      vs[2] = atof(text[3]);
		      ve[2] = atof(text[4]);
		    }
		  
		  if (strcasecmp(text[0], "Lambda") == 0)
		    {
		      *wavelength = atoi(text[2]);
		      *wavelength /= 1e4;
		      strcpy(ins_wavelength, text[2]);
		    }
		  
		  
		  //*** get the baselines for the channels
		  //0 = FIX
		  //1 = SD1
		  //2 = SD2
		  if ((strcasecmp(text[0], "Pixel") == 0) && (strcasecmp(text[1], "0") == 0))
		    {
		      if (strcasecmp(text[3], "FIX") == 0) ch_baseline[0] = 0;
		      if (strcasecmp(text[3], "SD1") == 0) ch_baseline[0] = 1;
		      if (strcasecmp(text[3], "SD2") == 0) ch_baseline[0] = 2;
		    }
		  
		  if ((strcasecmp(text[0], "Pixel") == 0) && (strcasecmp(text[1], "1") == 0))
		    {
		      if (strcasecmp(text[4], "FIX") == 0) ch_baseline[1] = 0;
		      if (strcasecmp(text[4], "SD1") == 0) ch_baseline[1] = 1;
		      if (strcasecmp(text[4], "SD2") == 0) ch_baseline[1] = 2;
		    }
		  
		  if ((strcasecmp(text[0], "Pixel") == 0) && (strcasecmp(text[1], "2") == 0))
		    {
		      if (strcasecmp(text[3], "FIX") == 0) ch_baseline[2] = 0;
		      if (strcasecmp(text[3], "SD1") == 0) ch_baseline[2] = 1;
		      if (strcasecmp(text[3], "SD2") == 0) ch_baseline[2] = 2;
		    }
		  
		  if ((strcasecmp(text[0], "Pixel") == 0) && (strcasecmp(text[1], "3") == 0))
		    {
		      if (strcasecmp(text[4], "FIX") == 0) ch_baseline[3] = 0;
		      if (strcasecmp(text[4], "SD1") == 0) ch_baseline[3] = 1;
		      if (strcasecmp(text[4], "SD2") == 0) ch_baseline[3] = 2;
		    }
		  
		  if ((strcasecmp(text[0], "Pixel") == 0) && (strcasecmp(text[1], "4") == 0))
		    {
		      if (strcasecmp(text[3], "FIX") == 0) ch_baseline[4] = 0;
		      if (strcasecmp(text[3], "SD1") == 0) ch_baseline[4] = 1;
		      if (strcasecmp(text[3], "SD2") == 0) ch_baseline[4] = 2;
		    }
		  
		  if ((strcasecmp(text[0], "Pixel") == 0) && (strcasecmp(text[1], "5") == 0))
		    {
		      if (strcasecmp(text[4], "FIX") == 0) ch_baseline[5] = 0;
		      if (strcasecmp(text[4], "SD1") == 0) ch_baseline[5] = 1;
		      if (strcasecmp(text[4], "SD2") == 0) ch_baseline[5] = 2;
		    }

		  //0 = A
		  //1 = B
		  //2 = C
		  if ((strcasecmp(text[0], "Fixed") == 0) && (strcasecmp(text[1], "Delay") == 0))
		    {
		      if (strcasecmp(text[3], "A") == 0) baselines[0] = 0;
		      if (strcasecmp(text[3], "B") == 0) baselines[0] = 1;
		      if (strcasecmp(text[3], "C") == 0) baselines[0] = 2;
		    }

		  if ((strcasecmp(text[0], "Short") == 0) && (strcasecmp(text[2], "1") == 0))
		    {
		      if (strcasecmp(text[4], "A") == 0) baselines[1] = 0;
		      if (strcasecmp(text[4], "B") == 0) baselines[1] = 1;
		      if (strcasecmp(text[4], "C") == 0) baselines[1] = 2;
		    }
		  
		  if ((strcasecmp(text[0], "Short") == 0) && (strcasecmp(text[2], "2") == 0))
		    {
		      if (strcasecmp(text[4], "A") == 0) baselines[2] = 0;
		      if (strcasecmp(text[4], "B") == 0) baselines[2] = 1;
		      if (strcasecmp(text[4], "C") == 0) baselines[2] = 2;
		    }


		  if ((strcasecmp(text[0], "Fringe") == 0) && (strcasecmp(text[1], "0") == 0) && (strcasecmp(text[2], "Freq") == 0)) fringe_freq[0] = atof(text[5]);
		  if ((strcasecmp(text[0], "Fringe") == 0) && (strcasecmp(text[1], "1") == 0) && (strcasecmp(text[2], "Freq") == 0)) fringe_freq[1] = atof(text[5]);
		  if ((strcasecmp(text[0], "Fringe") == 0) && (strcasecmp(text[1], "2") == 0) && (strcasecmp(text[2], "Freq") == 0)) fringe_freq[2] = atof(text[5]);
		  if ((strcasecmp(text[0], "Fringe") == 0) && (strcasecmp(text[1], "3") == 0) && (strcasecmp(text[2], "Freq") == 0)) fringe_freq[3] = atof(text[5]);
		  if ((strcasecmp(text[0], "Fringe") == 0) && (strcasecmp(text[1], "4") == 0) && (strcasecmp(text[2], "Freq") == 0)) fringe_freq[4] = atof(text[5]);
		  if ((strcasecmp(text[0], "Fringe") == 0) && (strcasecmp(text[1], "5") == 0) && (strcasecmp(text[2], "Freq") == 0)) fringe_freq[5] = atof(text[5]);
		}
	    }
	  
	  for (n=0; n<6; n++)
	    ch_baseline[n] = baselines[ch_baseline[n]];

	  for (n=0; n<3; n++)
	  {
	      ustart[n] = us[n];
	      uend[n] = ue[n];
	      vstart[n] = vs[n];
	      vend[n] = ve[n];
	  }

	  strcpy(filename, ProcDir);
	  strcat(filename, "objid.ini");

	  strcpy(filename2, ProcDir);
	  strcat(filename2, "calib.ini");
	  
	  found = 0;
	  listcount = 0;

	  if (file_exist(filename) == 1)
	  {
	      stream2 = fopen(filename,"r");
	      
	      //change IOTA-star-id to object ID
	      while (feof(stream2) == 0)
	      {
		  fscanf(stream2, "%s %s %s %i\n", &oldid, &text1, &text2, &newid);
		  
		  if (strcasecmp(objid_, oldid) == 0)
		  {	
		      found = 1;
		      *objid = newid;
		  }
		 
		  strcpy(objidlist[listcount], oldid);
		  objidlistnr[listcount] = newid;
		  listcount++;
	      }
	      
	      fclose(stream2);
	  }

	  if ((found == 0) && (autogetobjid == 1))
	  {
	      count = -1;
	      objidnew = 0;

	      while (objidnew == 0)
	      {
		  count++;
		  found = 0;
		  
		  for (count2 = 0; count2 < listcount; count2++)
		  {
		      if (count == objidlistnr[count2]) found = 1;
		  }
		  
		  if (found == 0)
		      objidnew = count;
	      }
	      
	      strcpy(objidlist[listcount], objid_);
	      objidlistnr[listcount] = objidnew;

	      printf("\n\n=============================================================\n");
	      printf("           NEW OBJECT: %s -> ID #%i\n", objidlist[listcount], objidlistnr[listcount]);
	      printf("=============================================================\n");

	      c = 'a';
	      while ((c != 'c') && (c != 'C') && (c != 't') && (c != 'T') && (c != 'r') && (c != 'R'))
	      {
		  printf("\n");
		  printf("Please specify object type: TARGET ('T'), CALIBRATOR ('C') or RESOLVED CALIBRATOR ('R')?\nType 'T', 'C' or 'R' and press ENTER: ");

		  scanf("%c", &c);
	      }
	      scanf("%c", &c2);
	      printf("\n");

	      //add object to calibrator list
	      if ((c == 'c') || (c == 'C'))
	      {
		  for (count2 = 0; count2 <= 9; count2++)
		  {
		      clist[count2] = 0;
		      rlist[count2] = 0;
		      rdlist[count2] = 0.0;
		  }

		  if (file_exist(filename2) == 1)
		  {
		      stream2 = fopen(filename2,"r");
		      
		      fscanf(stream2, "Calibrators:   %i %i %i %i %i %i %i %i %i\n", &clist[0], &clist[1], &clist[2], &clist[3], &clist[4], &clist[5], &clist[6], &clist[7], &clist[8], &clist[9]);
		      fscanf(stream2, "ResolvedCalibrators:   %i %i %i %i %i %i %i %i %i\n", &rlist[0], &rlist[1], &rlist[2], &rlist[3], &rlist[4], &rlist[5], &rlist[6], &rlist[7], &rlist[8], &rlist[9]);
		      fscanf(stream2, "ResolvedCalibratorsDiameters:   %f %f %f %f %f %f %f %f %f\n", &rdlist[0], &rdlist[1], &rdlist[2], &rdlist[3], &rdlist[4], &rdlist[5], &rdlist[6], &rdlist[7], &rdlist[8], &rdlist[9]);
		      
		      fclose(stream2);
		  }

		  found = 0;
		  for (count2 = 0; count2 <= 9; count2++)
		      if ((clist[count2] == 0) && (found == 0))
		      {
			  clist[count2] = objidlistnr[listcount];
			  found = 1;
		      }


		  stream2 = fopen(filename2,"w");
		  
		  fprintf(stream2, "Calibrators:   %i %i %i %i %i %i %i %i %i\n", clist[0], clist[1], clist[2], clist[3], clist[4], clist[5], clist[6], clist[7], clist[8], clist[9]);
		  fprintf(stream2, "ResolvedCalibrators:   %i %i %i %i %i %i %i %i %i\n", rlist[0], rlist[1], rlist[2], rlist[3], rlist[4], rlist[5], rlist[6], rlist[7], rlist[8], rlist[9]);
		  fprintf(stream2, "ResolvedCalibratorsDiameters:   %f %f %f %f %f %f %f %f %f\n", rdlist[0], rdlist[1], rdlist[2], rdlist[3], rdlist[4], rdlist[5], rdlist[6], rdlist[7], rdlist[8], rdlist[9]);
		  
		  fclose(stream2);
	      }


	      if ((c == 'r') || (c == 'R'))
	      {
		  printf("Specify calibrator diameter [mas]: ");
		  scanf("%f", &rdiameter);
		  scanf("%c", &c2);
		  printf("\n");

		  for (count2 = 0; count2 <= 9; count2++)
		  {
		      clist[count2] = 0;
		      rlist[count2] = 0;
		      rdlist[count2] = 0.0;
		  }

		  if (file_exist(filename2) == 1)
		  {
		      stream2 = fopen(filename2,"r");
		      
		      fscanf(stream2, "Calibrators:   %i %i %i %i %i %i %i %i %i\n", &clist[0], &clist[1], &clist[2], &clist[3], &clist[4], &clist[5], &clist[6], &clist[7], &clist[8], &clist[9]);
		      fscanf(stream2, "ResolvedCalibrators:   %i %i %i %i %i %i %i %i %i\n", &rlist[0], &rlist[1], &rlist[2], &rlist[3], &rlist[4], &rlist[5], &rlist[6], &rlist[7], &rlist[8], &rlist[9]);
		      fscanf(stream2, "ResolvedCalibratorsDiameters:   %f %f %f %f %f %f %f %f %f\n", &rdlist[0], &rdlist[1], &rdlist[2], &rdlist[3], &rdlist[4], &rdlist[5], &rdlist[6], &rdlist[7], &rdlist[8], &rdlist[9]);
		      
		      fclose(stream2);
		  }

		  found = 0;
		  for (count2 = 0; count2 <= 9; count2++)
		      if ((rlist[count2] == 0) && (found == 0))
		      {
			  rlist[count2] = objidlistnr[listcount];
			  rdlist[count2] = rdiameter;
			  found = 1;
		      }

		  stream2 = fopen(filename2,"w");
		  
		  fprintf(stream2, "Calibrators:   %i %i %i %i %i %i %i %i %i\n", clist[0], clist[1], clist[2], clist[3], clist[4], clist[5], clist[6], clist[7], clist[8], clist[9]);
		  fprintf(stream2, "ResolvedCalibrators:   %i %i %i %i %i %i %i %i %i\n", rlist[0], rlist[1], rlist[2], rlist[3], rlist[4], rlist[5], rlist[6], rlist[7], rlist[8], rlist[9]);
		  fprintf(stream2, "ResolvedCalibratorsDiameters:   %f %f %f %f %f %f %f %f %f\n", rdlist[0], rdlist[1], rdlist[2], rdlist[3], rdlist[4], rdlist[5], rdlist[6], rdlist[7], rdlist[8], rdlist[9]);
		  
		  fclose(stream2);
	      }

	      *objid = objidlistnr[listcount];

	      //write to file
	      stream2 = fopen(filename,"w");

	      for (count2 = 0; count2 <= listcount; count2++)
		  fprintf(stream2, "%s -> ID %i\n", objidlist[count2], objidlistnr[count2]);

	      fclose(stream2);
	  }
	}
      
      strcpy(arr_name, "");
      strcat(arr_name, "IOTA_");
      strncat(arr_name, sta_tel0, 4);
      strncat(arr_name, sta_tel1, 4);
      strncat(arr_name, sta_tel2, 4);

      strcpy(ins_name, "");
      strcat(ins_name, ins_combiner);
      strcat(ins_name, "_");
      strcat(ins_name, ins_wavelength);

      fprintf(streaminfo, "%i %i  %s %s %s  %s %lf %lf %lf   %s %s %s  %s %s %s %s   %s %s %s   %s %s %s   %s %s %s\n", scriptnrall, *objid, tar_star, tar_ra, tar_dec, vis_date, vis_time, vis_mjd, vis_int_time, ins_name, ins_combiner, ins_wavelength, arr_name, sta_tel0, sta_tel1, sta_tel2, sta_tel0_0, sta_tel0_1, sta_tel0_2, sta_tel1_0, sta_tel1_1, sta_tel1_2, sta_tel2_0, sta_tel2_1, sta_tel2_2);

      fclose(stream);
    } 
}

// Simple swap function needed for fourn function.
#define IOTASWAP(a,b) tempr=(a);(a)=(b);(b)=tempr

void fourn(double *data, int *nn, int ndim, int isign)
{

    int i1,i2,i3,i2rev,i3rev,ip1,ip2,ip3,ifp1,ifp2;
    int ibit,idim,k1,k2,n,nprev,nrem,ntot;
	// defined these variables to be same type as ArrayData
    double tempi,tempr;
    double theta,wi,wpi,wpr,wr,wtemp;

    ntot=1;
    for (idim=1;idim<=ndim;idim++)
        ntot *= nn[idim];
    nprev=1;
    for (idim=ndim;idim>=1;idim--) 
    {
        n=nn[idim];
        nrem=ntot/(n*nprev);
        ip1=nprev << 1;
        ip2=ip1*n;
        ip3=ip2*nrem;
        i2rev=1;
        for (i2=1;i2<=ip2;i2+=ip1) 
        {
            if (i2 < i2rev) 
            {
                for (i1=i2;i1<=i2+ip1-2;i1+=2) 
                { 
                    for (i3=i1;i3<=ip3;i3+=ip2) 
                    {
                        i3rev=i2rev+i3-i2;
                        IOTASWAP(data[i3],data[i3rev]);
                        IOTASWAP(data[i3+1],data[i3rev+1]);
                    }
                }
            }
            ibit=ip2 >> 1;
            while (ibit >= ip1 && i2rev > ibit) 
            {
                i2rev -= ibit;
                ibit >>= 1;
            }
            i2rev += ibit;
        }
        ifp1=ip1;
        while (ifp1 < ip2) 
        {
            ifp2=ifp1 << 1;
            theta=isign*6.28318530717959/(ifp2/ip1);
            wtemp=sin(0.5*theta);
            wpr = -2.0*wtemp*wtemp;
            wpi=sin(theta);
            wr=1.0;
            wi=0.0;
            for (i3=1;i3<=ifp1;i3+=ip1) 
            {
                for (i1=i3;i1<=i3+ip1-2;i1+=2) 
                {
                    for (i2=i1;i2<=ip3;i2+=ifp2) 
                    {
                        k1=i2;
                        k2=k1+ifp1;
                        tempr=wr*data[k2]-wi*data[k2+1];
                        tempi=wr*data[k2+1]+wi*data[k2];
                        data[k2]=data[k1]-tempr;
                        data[k2+1]=data[k1+1]-tempi;
                        data[k1] += tempr;
                        data[k1+1] += tempi;
                    }
                }
                wr=(wtemp=wr)*wpr-wi*wpi+wr;
                wi=wi*wpr+wtemp*wpi+wi;
            }
            ifp1=ifp2;
        }
        nprev *= n;
    }
}

#undef IOTASWAP


void fft(double *d, int *nn)
{
  double *dp, sum;
  int *nnp;
  int i;

  /* remove offset */
  sum = 0.0;
  for(i=0; i<nn[0]; i++)
    sum += d[i];
  sum /= nn[0];

  for(i=0; i<nn[0]; i++)
    d[i] -= sum;

  dp = d-1;
  nnp = nn-1;
  fourn(dp, nnp, 1, -1);
}


int idx(int *nn, int i, int j, int k)
{
	int offset,i2,j2,k2;

	//Always using at least 1st index.

	if(i<nn[0]/2)
	{
		i2 = nn[0]/2 + i;
	}
	else
	{
		i2 = nn[0]/2 + i - nn[0];
	}

	if (j == -1)
	{
		j2 = 0;
	}
	else
	{
		if(j<nn[1]/2)
		{
			j2 = nn[1]/2 + j;
		}
		else
		{
			j2 = nn[1]/2 + j - nn[1];
		}
	}

	if (k == -1)
	{
		k2 = 0;//Not using 3rd index at all.
	}
	else  //Using 3rd index.
	{
		if(k<=nn[2]/2)
		{
			k2 = nn[2]/2 + k;
		}
		else
		{
			k2 = nn[2]/2 + k - nn[2];
		}
	}

	offset = i2 + j2*nn[0] + k2*nn[0]*nn[1];
	return(2*offset); //Multiply by 2 to skip complex values.
}




int read_ConfigFile(char *file)
{
    FILE *stream;
    int i, j, notfound;
    char text_[300];
    char text[10][300];

    printf("Configuration File: %s\n", file);
    printf("Read Entries:\n");

    if (file_exist(file))
    {
	stream = fopen(file, "r");

	i = 0;
	if (stream != NULL)
	    while (feof(stream) == 0)
	    {
		i++;
		
		//read whole line in text_
		j = 0;
		for (;;)
		{
		    text_[j]  = getc(stream);
		    if (text_[j] == 10) break;
		    if (feof(stream) != 0) break;
		    if (j > 298) break;
		    j++;
		}

		//separate words in text[0]
		if (j > 10)
		{
		    sscanf(text_, "%s %s %s %s %s %s %s %s %s %s", text[0], text[1], text[2], text[3], text[4], text[5], text[6], text[7], text[8], text[9]);

		    if ((text[0][0] != 40) && (text[0][0] != 42))
		    {
			printf("     %40s = %s", text[0], text[1]);
			notfound = 1;
			
			//General
			if (strcasecmp(text[0], "ReportDetailsScreen") == 0)
			{
			    repdetails = atoi(text[1]);
			    notfound = 0;
			}
			
			if (strcasecmp(text[0], "PlotPC") == 0)
			{
			    doplot0 = atoi(text[1]);
			    notfound = 0;
			}
			if (strcasecmp(text[0], "PlotRaw") == 0)
			{
			    doplot1 = atoi(text[1]);
			    notfound = 0;
			}
			if (strcasecmp(text[0], "PlotProc") == 0)
			{
			    doplot2 = atoi(text[1]);
			    notfound = 0;
			}
			if (strcasecmp(text[0], "PlotHisto") == 0)
			{
			    doplot3 = atoi(text[1]);
			    notfound = 0;
			}
			if (strcasecmp(text[0], "PlotBG") == 0)
			{
			    doplotbg = atoi(text[1]);
			    notfound = 0;
			}
			if (strcasecmp(text[0], "PlotWait") == 0)
			{
			    doplotwait = atoi(text[1]);
			    notfound = 0;
			}
			
			//Visibility Estimation
			if (strcasecmp(text[0], "VisEstimationMethod") == 0)
			{
			    if (atoi(text[1]) == 0) fitampmethod = 2;
			    if (atoi(text[1]) == 1) fitampmethod = 1;
			    notfound = 0;
			}
			if (strcasecmp(text[0], "VisCalibrationMethod") == 0)
			{
			    Viscalibmethod = atoi(text[1]);
			    notfound = 0;
			}
			if (strcasecmp(text[0], "VisSubBackground") == 0)
			{
			    dobackgroundsub = atoi(text[1]);
			    notfound = 0;
			}
			if (strcasecmp(text[0], "VisSubBackgroundSmooth") == 0)
			{
			    bgsubsmoothfactor = atof(text[1]);
			    notfound = 0;
			}
			if (strcasecmp(text[0], "VisPhotometricCorrection") == 0)
			{
			    dophotcorr = atoi(text[1]);
			    notfound = 0;
			}
			if (strcasecmp(text[0], "VisPhotometricCorrectionSmooth1") == 0)
			{
			    photcorrsmoothfactor1 = atof(text[1]);
			    notfound = 0;
			}
			if (strcasecmp(text[0], "VisPhotometricCorrectionSmooth2") == 0)
			{
			    photcorrsmoothfactor2 = atof(text[1]);
			    notfound = 0;
			}
			
			if (strcasecmp(text[0], "VisErrorAddCalibrationError") == 0)
			{
			    addcaliberror = atoi(text[1]);
			    notfound = 0;
			}
			if (strcasecmp(text[0], "VisErrorAddSys") == 0)
			{
			    adderrorVis = atof(text[1]);		      
			    notfound = 0;
			}
			
			if (strcasecmp(text[0], "VisFilterRaw") == 0)
			{
			    dofilteramp = atoi(text[1]);
			    notfound = 0;
			}
			if (strcasecmp(text[0], "VisFilterRawThreshold") == 0)
			{
			    filterampthreshold = atof(text[1]);		      
			    notfound = 0;
			}
			
			if (strcasecmp(text[0], "VisFilterScatteringGroups") == 0)
			{
			    dofilterscat = atoi(text[1]);
			    notfound = 0;
			}
			if (strcasecmp(text[0], "VisFilterScatteringGroupsThreshold") == 0)
			{
			    filtervisscatthreshold = atof(text[1]);
			    notfound = 0;
			}
			
			if (strcasecmp(text[0], "VisFilterLargeErrorsThreshold") == 0)
			{
			    filterviserror = atof(text[1]);
			    notfound = 0;
			}
			
			if (strcasecmp(text[0], "VisFringeTrackerSelection") == 0)
			{
			    VisFringeTrackerSelection = atoi(text[1]);
			    notfound = 0;
			}
			
			if (strcasecmp(text[0], "VisFringeTrackerThreshold") == 0)
			{
			    FringeTrackerthresholdRatio = atof(text[1]);
			    notfound = 0;
			}
			
			//Vis - PS
			if (strcasecmp(text[0], "PSBackgroundBorder") == 0)
			{
			    powmaxrange = atoi(text[1]);
			    notfound = 0;
			}

			//Vis - CWT
			if (strcasecmp(text[0], "CWTPowerThreshold") == 0)
			{
			    cwt_filter_threshold_power = atof(text[1]);
			    notfound = 0;
			}
			if (strcasecmp(text[0], "CWTPowerThreshold") == 0)
			{
			    cwt_filter_threshold_filter = atof(text[1]);
			    notfound = 0;
			}
			
			if (strcasecmp(text[0], "CWTSubBackground") == 0)
			{
			    cwt_subtract_background = atoi(text[1]);
			    notfound = 0;
			}
			if (strcasecmp(text[0], "CWTBackgroundBorder") == 0)
			{
			    cwt_background_border = atoi(text[1]);
			    notfound = 0;
			}

			if (strcasecmp(text[0], "CWTThresholdAreaMin") == 0)
			{
			    cwt_threshold_area_min = atoi(text[1]);
			    notfound = 0;
			}
			if (strcasecmp(text[0], "CWTThresholdAreaMax") == 0)
			{
			    cwt_threshold_area_max = atoi(text[1]);
			    notfound = 0;
			}
			if (strcasecmp(text[0], "CWTThresholdIntPeakOPD") == 0)
			{
			    cwt_threshold_pos_intpeak_spatial = atoi(text[1]);
			    notfound = 0;
			}
			if (strcasecmp(text[0], "CWTThresholdIntPeakScale") == 0)
			{
			    cwt_threshold_pos_intpeak_freq = atoi(text[1]);
			    notfound = 0;
			}
			if (strcasecmp(text[0], "CWTThresholdSizePeakScaleCh0Ch1") == 0)
			{
			    cwt_threshold_size_peak_freq01 = atoi(text[1]);
			    notfound = 0;
			}
			if (strcasecmp(text[0], "CWTThresholdSizePeakScaleCh2") == 0)
			{
			    cwt_threshold_size_peak_freq2 = atoi(text[1]);
			    notfound = 0;
			}
			if (strcasecmp(text[0], "CWTThresholdSignal2Noise") == 0)
			{
			    cwt_threshold_SignalToNoise = atof(text[1]);
			    notfound = 0;
			}
			
			//CP
			if (strcasecmp(text[0], "CPCalibrationMethod") == 0)
			{
			    CPcalibmethod = atoi(text[1]);
			    notfound = 0;
			}
			if (strcasecmp(text[0], "CPWindowWidth") == 0)
			{
			    CPWindowWidth = atoi(text[1]);
			    notfound = 0;
			}
			if (strcasecmp(text[0], "CPFilterLargeErrorsThreshold") == 0)
			{
			    filtercperror = atof(text[1])*DToR;
			    notfound = 0;
			}
			if (strcasecmp(text[0], "CPErrorAddSys") == 0)
			{
			    adderrorCP = atof(text[1])*DToR;
			    notfound = 0;
			}
			if (strcasecmp(text[0], "CPFreqClosureThreshold") == 0)
			{
			    CPfreqclosureThreshold = atoi(text[1]);
			    notfound = 0;
			}

			if (notfound == 1)
			    printf("    INVALID ENTRY!!!\n");
			else
			    printf("\n");
		    }
		}
	    }

	printf("\n");
	return 1;
    }
    else
    {
	printf("ERROR: Configuration File '%s' not found!\n", file);
	return 0;
    }
};
