/* mathfunctions.c - standard math functions not included already in math.h

	changeSign_vector
	concatenate_lvector
	copy_lvector
	copy_vector
	dft_bin_nocomplex
	fix
	maximum_vector
	maximum_vectors
	mean_vector
	mult_scalar_vector
	print_lvector
	print_vector
	remainder
	roundFloat
	roundOdd
	shellsort_long
	sinc
	subtract_vector
	sum_vector
	zero_vector

  other functions already written and stored in oldfunctions.c:
  	add_vector
	concatenate_ivector
	copy_ivector
	interpolateZeros
	print_ivector
	shellsort_float
	shellsort_int
*/

#include <math.h>
#include <stdio.h>
#include "mathfunctions.h"
//#include "nrutil.h"
#include "trackerdefines.h"

#define PI 3.1415926535

/****************************************************
* vector change sign
* in MATLAB syntax, this would do:
*   y(nl:nh) = -x(nl:nh)
****************************************************/
void changeSign_vector(float *x, float *y, unsigned long nl, unsigned long nh)
{
	unsigned long i;

	/* offset vectors to account for nl */
	x = x + nl;
	y = y + nl;
    for (i = nl; i <= nh; i++)
		*y++ = -(*x++);
}

/****************************************************
* unsigned-long-vector concatenate
* in MATLAB syntax, this would do:
*   z(nl_z:nh_x-nl_x+nh_y-nl_y+2) = [x(nl_x:nh_x) y(nl_y:nh_y)]
****************************************************/
void concatenate_lvector(unsigned long *x, unsigned long *y, unsigned long *z, unsigned long nl_x, 
	unsigned long nh_x, unsigned long nl_y, unsigned long nh_y, unsigned long nl_z)
{
	unsigned long i;

	/* offset vectors to account for nl_x,y,z */
	x = x + nl_x;
	y = y + nl_y;
	z = z + nl_z;
    for (i = nl_x; i <= nh_x; i++)
		*z++ = *x++;
    for (i = nl_y; i <= nh_y; i++)
		*z++ = *y++;
}

/****************************************************
* unsigned-long-vector copy
* in MATLAB syntax, this would do:
*   y(nl:nh) = x(nl:nh)
****************************************************/
void copy_lvector(unsigned long *x, unsigned long *y, unsigned long nl, unsigned long nh)
{
	unsigned long i;

	/* offset vectors to account for nl */
	x = x + nl;
	y = y + nl;
    for (i = nl; i <= nh; i++)
		*y++ = *x++;
}

/****************************************************
* float-vector copy
* in MATLAB syntax, this would do:
*   y(nl:nh) = x(nl:nh)
****************************************************/
void copy_vector(float *x, float *y, unsigned long nl, unsigned long nh)
{
	unsigned long i;

	/* offset vectors to account for nl */
	x = x + nl;
	y = y + nl;
    for (i = nl; i <= nh; i++)
		*y++ = *x++;
}

/****************************************************
* dft_bin_nocomplex
* This calculates the DFT at a single bin frequency (corresponding to binNumber)
*	for an input vector x(nl:nh), and returns the result as vector y(nl:nh)
* The DFT at each point is calculated using a relatively small window containing 
*	nDFT neighboring samples.
* The computations are implemented very efficiently. For example, as the window
*	is passed over x, computations involve only adding the point coming into the 
*	window and subtracting the point leaving it.
* Code was originally implemented using complex numbers, but was converted so it
*	could be implemented using floats (rather than complex data types).
****************************************************/
void dft_bin_nocomplex(float *x, float *y, unsigned long nl, unsigned long nh, int nDFT, int binNumber)
{
	unsigned long k, i_num, N, N_DFT_2;
	float theta_k, *X_n_real, *X_n_imag, *X_real, *X_imag, temp;

	X_n_real = vector(nl,nh);
	X_n_imag = vector(nl,nh);
	X_real = vector(nl,nh);
	X_imag = vector(nl,nh);

	N = nh - nl + 1;

	if (remainder2((float) nDFT,2.0) == 0.0) {
		/* it's even, so change it to be odd */
		nDFT--;
	}
	N_DFT_2 = (unsigned long) floor(nDFT/2.0);

	temp = (float)(-2.0*PI*(binNumber-1.0)/nDFT);
	for (k=nl; k<=nh; k++) {
		theta_k = temp*(float)(k-nl);
		X_n_real[k] = x[k] * (float) cos(theta_k);
		X_n_imag[k] = x[k] * (float) sin(theta_k);
	}

	i_num = N_DFT_2 + 1;

	temp = mean_vector(X_n_real,nl,nl+i_num-1);
	X_real[nl] = temp * nDFT;
	temp = mean_vector(X_n_imag,nl,nl+i_num-1);
	X_imag[nl] = temp * nDFT;

	/* these values are set to zero below, and this code could be omitted (with modifications
	* to other code) if pressed for compute time. */
	for (k=nl+1; k<=N_DFT_2+nl; k++) {
		i_num++;
		X_real[k] = ((float) (i_num-1.0)*X_real[k-1] + nDFT*X_n_real[k+N_DFT_2])/(float)i_num;
		X_imag[k] = ((float) (i_num-1.0)*X_imag[k-1] + nDFT*X_n_imag[k+N_DFT_2])/(float)i_num;
	}

	for (k=N_DFT_2+nl+1; k<=N-N_DFT_2+nl-1; k++) {
		X_real[k] = X_real[k-1] + X_n_real[k+N_DFT_2] - X_n_real[k-N_DFT_2-1];
		X_imag[k] = X_imag[k-1] + X_n_imag[k+N_DFT_2] - X_n_imag[k-N_DFT_2-1];
	}
	/* following is not needed, since these points are set to zero below
	* i_num = nDFT - 1;
	* for (k=N-N_DFT_2+nl; k<=nh; k++) {
	* 	X_real[k] = ((float) (i_num+1.0)*X_real[k-1] - nDFT*X_n_real[k-N_DFT_2-1])/(float)i_num;
	* 	X_imag[k] = ((float) (i_num+1.0)*X_imag[k-1] - nDFT*X_n_imag[k-N_DFT_2-1])/(float)i_num;
	* 	i_num--;
	* } */

	/* calculate the magnitude of the DFT for all points calculated using the full
	* N_DFT data points. */
	for (k=N_DFT_2+nl; k<=N-N_DFT_2+nl-1; k++) {
		y[k] = (float) sqrt(SQR(X_real[k]) + SQR(X_imag[k]));
	}

	/* for the leading points (calculated with less than N_DFT data points), set
	* them equal to zero. */
	for (k=nl; k<N_DFT_2+nl; k++) {
		y[k] = 0.0; /* y[N_DFT_2+nl]; */
	}

	/* for the trailing points (calculated with less than N_DFT data points), set
	* them equal to zero. */
	for (k=N-N_DFT_2+nl; k<=nh; k++) {
		y[k] = 0.0; /* y[N-N_DFT_2+nl-1]; */
	}

	free_vector(X_n_real,nl,nh);
	free_vector(X_n_imag,nl,nh);
	free_vector(X_real,nl,nh);
	free_vector(X_imag,nl,nh);
}

/****************************************************
* round towards zero
* in MATLAB syntax, this would do:
* 	y = fix(x)
****************************************************/
float fix(float x)
{
	float y;

	if (x >= 0.0) {
		y = (float) floor(x);
	}
	else {
		y = (float) ceil(x);
	}
	return y;
}

/****************************************************
* find the index of the maximum value in a vector
* in MATLAB syntax, this would do:
*   [junk,n_max] = max(x(nl:nh))
****************************************************/
unsigned long maximum_vector(float *x, unsigned long nl, unsigned long nh)
{
	unsigned long i, i_max;
	float x_max;

	/* start by setting as if the first element in x were the maximum */
	x_max = x[nl];
	i_max = nl;

	for (i = nl+1; i <= nh; i++) {
		if (x[i] > x_max) {
			x_max = x[i];
			i_max = i;
		}
	}
	return i_max;
}

/****************************************************
* returns the higher value from two vectors
*   result is stored in the location held by the first input vector
* in MATLAB syntax, this would do:
*   x(nl:nh) = max([x(nl:nh); y(nl:nh)])
****************************************************/
void maximum_vectors(float *x, float *y, unsigned long nl, unsigned long nh)
{
	unsigned long i;

	for (i = nl; i <= nh; i++) {
		if (y[i] > x[i]) {
			x[i] = y[i];
		}
	}
}

/****************************************************
* calculate mean of a vector
* in MATLAB syntax, this would do:
*   ave = mean(x(nl:nh))
****************************************************/
float mean_vector(float *x, unsigned long nl, unsigned long nh)
{
	unsigned long j;
	float sum_x = 0.0, ave;

	for (j=nl; j<=nh; j++)
		sum_x += x[j];
	ave = sum_x / (float) (nh-nl+1.0);

	return ave;
}

/****************************************************
* vector multiplication by a scalar
* in MATLAB syntax, this would do:
*   z(nl:nh) = x(nl:nh) * y
****************************************************/
void mult_scalar_vector(float *x, float y, float *z, unsigned long nl, unsigned long nh)
{
	unsigned long i;

	/* offset vectors to account for nl > 0 */
	x = x + nl;
	z = z + nl;
    for (i = nl; i <= nh; i++)
		*z++ = (*x++) * (y);
}

/****************************************************
* prints a vector to the screen
* example syntax: 	print_vector(y_raw, 1, nPtsScan, "y_raw");
****************************************************/
void print_lvector(unsigned long *x, unsigned long nl, unsigned long nh, char vectorName[])
{
	unsigned long i;
	for (i=nl;i<=nh;i++) {
		printf("%s[%u] = %u\n", vectorName, i, x[i]);
	}
	printf("\n");
}

void print_vector(float *x, unsigned long nl, unsigned long nh, char vectorName[])
{
	unsigned long i;
	for (i=nl;i<=nh;i++) {
		printf("%s[%u] = %f\n", vectorName, i, x[i]);
	}
	printf("\n");
}

/****************************************************
* remainder after division
* in MATLAB syntax, this would do:
* 	y = rem(x)
****************************************************/
float remainder2(float x, float y)
{
	float z;

	if (y == 0.0) {
		z = 0; /* the MATLAB version returns NaN here */
	}
	else {
		z = x - y * fix(x/y);
	}
	return z;
}

/****************************************************
* rounds a float to the nearest integer
* in MATLAB syntax, this would do:
* 	y = round(x)
****************************************************/
int roundFloat(float x)
{
	int y;

	y = (int) floor(x);
	if ((x-y) >= 0.5)
		y++;

	return y;
}

/****************************************************
* rounds a float to the nearest odd integer
* in MATLAB syntax, this would do:
* 	y = 1 + (2 * round((x-1)/2))
****************************************************/
int roundOdd(float x)
{
	int y;

	y = 1 + (2 * roundFloat((float)((x-1.0)/2.0)));

	return y;
}

/****************************************************
* shell sort from NRinC
* a is the input vector, which is sorted (changed)
* n is the length of a
* assumes a starts at 1, so it sorts a(1:n)
****************************************************/
void shellsort_long(unsigned long n, unsigned long a[])
{
	unsigned long i,j,inc;
	unsigned long v;
	inc=1;
	do {
		inc *= 3;
		inc++;
	} while (inc <= n);
	do {
		inc /= 3;
		for (i=inc+1;i<=n;i++) {
			v=a[i];
			j=i;
			while (a[j-inc] > v) {
				a[j]=a[j-inc];
				j -= inc;
				if (j <= inc) break;
			}
			a[j]=v;
		}
	} while (inc > 1);
}

/****************************************************
* sinc function (sinc as defined by MATLAB)
* in MATLAB syntax, this would do:
*    y = sinc(c)
****************************************************/
float sinc(float x)
{
	float y;

	if ( x == 0)
		y = 1.0;
	else
		y = (float) (sin(PI * x) / (PI * x));

	return(y);
}

/****************************************************
* vector subtraction
* in MATLAB syntax, this would do:
*   z(nl:nh) = x(nl:nh) - y(nl:nh)
****************************************************/
void subtract_vector(float *x, float *y, float *z, unsigned long nl, unsigned long nh)
{
	unsigned long i;

	/* offset vectors to account for nl > 0 */
	x = x + nl;
	y = y + nl;
	z = z + nl;
    for (i = nl; i <= nh; i++)
		*z++ = (*x++) - (*y++);
}

/****************************************************
* calculate sum of a vector
* in MATLAB syntax, this would do:
*   sum_x = sum(x(nl:nh))
****************************************************/
float sum_vector(float *x, unsigned long nl, unsigned long nh)
{
	unsigned long j;
	float sum_x;

	for (sum_x=0.0,j=nl;j<=nh;j++)
		sum_x += x[j];

	return sum_x;
}

/****************************************************
* zero vector
* in MATLAB syntax, this would do:
*   y(nl:nh) = 0
****************************************************/
void zero_vector(float *x, unsigned long nl, unsigned long nh)
{
	unsigned long i;

	/* offset vector to account for nl */
	x = x + nl;
    for (i = nl; i <= nh; i++)
		*x++ = 0.0;
}

