/* model.c                                   */
/* Interferometry Data Reduction Software    */
/* by Stefan Kraus                           */
/* 2002-05                                   */
/*                                           */
/* copyright laws apply                      */

//Observatory parameters
#if (interf_array == 1) 
  //VLTI
  #define obs_lat         -24.62794830	     //[degree] Latitude, Breite
  #define obs_lon         -70.40479659       //[degree] Longitude, Laenge
#else
  //IOTA
  #define obs_lat	  31.688889	     //[degree] Latitude, Breite
  #define obs_lon         -110.88444444      //[degree] Longitude, Laenge
#endif

//Object parameter
#define pmRA		75.52			//[mas/yr]
#define pmDEC		-427.11			//[mas/yr]

//General constants
//#define PI	        3.14159265358979
#define earthyear	365.2563604
#define solarr          6.960E8                         //[m]
#define pc2m            3.086E16                        //[m]
#define	DToR	        (PI/180.0)
#define	RToD	        (180.0/PI)


//Modeling of the visibility of an uniform disc
#define d_min    0.0
#define d_max   50.0
#define d_step   0.01


float wavelength_;


float fsqr(float x)
{
    return (x*x);
}


//returns the visibility of a star with DIAMETER d [mas] at a baseline b and observed at a particular wavelength

float vis_unif_disc(float D, float wavel, float B)
{
  float x, v;

  x = PI * B * (D/1000.0/3600.0*DToR) / wavel;
  v = fabs(2.0*j1(x) / x);

  return v;
};


void func_unif_disc(float x, float a[], float *y, float dyda[], int na)
{
  float arg;

  arg = PI * x * (a[1]/1000.0/3600.0*DToR) / wavelength_;
  *y = fabs(2.0*j1f(arg) / arg);
  dyda[1] = -2.0*(jnf(2, arg) / arg) * (PI*x*(1.0/1000.0/3600.0*DToR) / wavelength_);
}


void fit_model_unif_disc2(float wavel, float vispoints[max_targets][max_vispoints][3], int target, int vispoints_nr, float *diameter, float *error)
{
    const int MA = 1;

    int i,*ia,iter,itst,k,mfit=MA, points, errormsg;
    float alamda,chisq,ochisq,*x,*y,*sig,**covar,**alpha;
    float extremmin, extremmax, extremmaxx;
    float xmin, xmax, ymin, ymax;
    float a[MA+1];
    float gues[MA+1];

    gues[0] = 0.0;
    gues[1] = 0.001;
  
  wavelength_ = wavel;

  points = 0;
  for (i=0; i<=vispoints_nr; i++)
    if (vispoints[target][i][1] > 0.0)
      points++;
  
  ia=ivector(1,MA);
  x=vector(1,points);
  y=vector(1,points);
  sig=vector(1,points);
  covar=matrix(1,MA,1,MA);
  alpha=matrix(1,MA,1,MA);
  
  points = 0;
  for (i=0; i<=vispoints_nr; i++)
    if (vispoints[target][i][1] > 0.0)
      {
	points++;
	x[points] = vispoints[target][i][0]/100.0;
	y[points] = vispoints[target][i][1];
	sig[points] = vispoints[target][i][2];
      }
  
  
  errormsg = 0;
  gues[1] = 2.0;
  
  
  for (i=1;i<=mfit;i++) ia[i]=1;
  for (i=1;i<=MA;i++) a[i]=gues[i];
  
  for (iter=1;iter<=2;iter++) {
    alamda = -1;
    mrqmin(x,y,sig,points,a,ia,MA,covar,alpha,&chisq,func_unif_disc,&alamda);
    k=1;
    itst=0;
    for (;;) {
      k++;
      ochisq=chisq;
      mrqmin(x,y,sig,points,a,ia,MA,covar,alpha,&chisq,func_unif_disc,&alamda);
      if (chisq > ochisq)
	itst=0;
      else if (fabs(ochisq-chisq) < 0.1)
	itst++;
      if (itst < 6) continue;
      alamda=0.0;
      mrqmin(x,y,sig,points,a,ia,MA,covar,alpha,&chisq,func_unif_disc,&alamda);
      
      break;
    }
  }
  
  
  free_matrix(alpha,1,MA,1,MA);
  free_matrix(covar,1,MA,1,MA);
  free_vector(sig,1,points);
  free_vector(y,1,points);
  free_vector(x,1,points);
  free_ivector(ia,1,MA);
  
  if (a[1] == gues[1]) errormsg = 1;
  
  
  if (errormsg != 0)
    {
      *diameter = -1.0;
      *error = -1.0;
    }
  else
    {
      *diameter = a[1];
      *error = sqrt(covar[1][1]);
    }
  
};



#undef d_min
#undef d_max
#undef d_step


double modPI(double x)
{
  int x_;

  x_ = (int)(x/(2.0*PI) + 0.5);

  return (x - 2.0*PI*x_);
}

double modPI2(double x)
{
  int x_;

  x_ = (int)(x/(2.0*PI));

  return (x - 2.0*PI*x_);
}

double dsqr(double x)
{
  return (x*x);
}



void baseline_vectors_vlti(int baseline, char staA_pos[5], char staB_pos[5], char staC_pos[5], double *BN_, double *BE_, double *BU_)
{
    float Bn[3], Be[3], Bu[3], Bd[3];
    int i;
    char station[5];
    
    for (i=0; i<3; i++)
    {
	if (i == 0) strcpy(station, staA_pos);
	if (i == 1) strcpy(station, staB_pos);
	if (i == 2) strcpy(station, staC_pos);

	Bu[i] = 0.0;
	Bd[i] = 0.0;
	
	if (strcasecmp(station, "A0") == 0)
	{
	    Bn[i] = -55.812;
	    Be[i] = -14.642;
	}

	if (strcasecmp(station, "A1") == 0)
	{
	    Bn[i] = -70.949;
	    Be[i] = -9.434;
	}

	if (strcasecmp(station, "B0") == 0)
	{
	    Bn[i] = -53.212;
	    Be[i] = -7.065;
	}

	if (strcasecmp(station, "B1") == 0)
	{
	    Bn[i] = -68.334;
	    Be[i] = -1.863;
	}

	if (strcasecmp(station, "B2") == 0)
	{
	    Bn[i] = -75.899;
	    Be[i] = 0.739;
	}

	if (strcasecmp(station, "B3") == 0)
	{
	    Bn[i] = -83.481;
	    Be[i] = 3.348;
	}

	if (strcasecmp(station, "B4") == 0)
	{
	    Bn[i] = -91.030;
	    Be[i] = 5.945;
	}

	if (strcasecmp(station, "B5") == 0)
	{
	    Bn[i] = -98.594;
	    Be[i] = 8.547;
	}

	if (strcasecmp(station, "C0") == 0)
	{
	    Bn[i] = -50.607;
	    Be[i] = 0.487;
	}

	if (strcasecmp(station, "C1") == 0)
	{
	    Bn[i] = -65.735;
	    Be[i] = 5.691;
	}

	if (strcasecmp(station, "C2") == 0)
	{
	    Bn[i] = -73.307;
	    Be[i] = 8.296;
	}

	if (strcasecmp(station, "C3") == 0)
	{
	    Bn[i] = -80.864;
	    Be[i] = 10.896;
	}

	if (strcasecmp(station, "D0") == 0)
	{
	    Bn[i] = -45.397;
	    Be[i] = 15.628;
	}

	if (strcasecmp(station, "D1") == 0)
	{
	    Bn[i] = -75.660;
	    Be[i] = 26.039;
	}

	if (strcasecmp(station, "D2") == 0)
	{
	    Bn[i] = -90.787;
	    Be[i] = 31.243;
	}

	if (strcasecmp(station, "E0") == 0)
	{
	    Bn[i] = -40.196;
	    Be[i] = 30.760;
	}

	if (strcasecmp(station, "G0") == 0)
	{
	    Bn[i] = -34.990;
	    Be[i] = 45.896;
	}

	if (strcasecmp(station, "G1") == 0)
	{
	    Bn[i] = -95.501;
	    Be[i] = 66.716;
	}

	if (strcasecmp(station, "G2") == 0)
	{
	    Bn[i] = -12.289;
	    Be[i] = 38.063;
	}

	if (strcasecmp(station, "H0") == 0)
	{
	    Bn[i] = -24.572;
	    Be[i] = 76.150;
	}

	if (strcasecmp(station, "I1") == 0)
	{
	    Bn[i] = -59.789;
	    Be[i] = 96.711;
	}

	if (strcasecmp(station, "J1") == 0)
	{
	    Bn[i] = -39.444;
	    Be[i] = 106.648;
	}

	if (strcasecmp(station, "J2") == 0)
	{
	    Bn[i] = -62.151;
	    Be[i] = 114.460;
	}

	if (strcasecmp(station, "J3") == 0)
	{
	    Bn[i] = 36.193;
	    Be[i] = 80.628;
	}

	if (strcasecmp(station, "J4") == 0)
	{
	    Bn[i] = 51.193;
	    Be[i] = 75.424;
	}

	if (strcasecmp(station, "J5") == 0)
	{
	    Bn[i] = 74.009;
	    Be[i] = 67.618;
	}

	if (strcasecmp(station, "J6") == 0)
	{
	    Bn[i] = 96.706;
	    Be[i] = 59.810;
	}

	if (strcasecmp(station, "K0") == 0)
	{
	    Bn[i] = -14.165;
	    Be[i] = 106.397;
	}

	if (strcasecmp(station, "L0") == 0)
	{
	    Bn[i] = -11.549;
	    Be[i] = 113.977;
	}

	if (strcasecmp(station, "M0") == 0)
	{
	    Bn[i] = -8.951;
	    Be[i] = 121.535;
	}

	if (strcasecmp(station, "U1") == 0)
	{
	    Bn[i] = -20.335;
	    Be[i] = -9.925;
	}

	if (strcasecmp(station, "U2") == 0)
	{
	    Bn[i] = 30.502;
	    Be[i] = 14.887;
	}

	if (strcasecmp(station, "U3") == 0)
	{
	    Bn[i] = 66.183;
	    Be[i] = 44.915;
	}

	if (strcasecmp(station, "U4") == 0)
	{
	    Bn[i] = 43.999;
	    Be[i] = 103.306;
	}
    }
    
    //*** calculate Baselines for AB, AC, BC
    if (baseline == 0) //AB
    {
	*BE_ = Be[0]-Be[1];
	*BN_ = Bn[0]-Bn[1];
	*BU_ = Bu[0]-Bu[1];
    }
    
    if (baseline == 1) //AC
    {
	*BE_ = Be[0]-Be[2];
	*BN_ = Bn[0]-Bn[2];
	*BU_ = Bu[0]-Bu[2];
    }
    
    if (baseline == 2) //BC
    {
	*BE_ = Be[1]-Be[2];
	*BN_ = Bn[1]-Bn[2];
	*BU_ = Bu[1]-Bu[2];
    }
	
    //to make it match with the header-file (u, v)'s and meter -> centimeter
    *BE_ *= 100.0;
    *BN_ *= 100.0;
    *BU_ *= 100.0;
}


void baseline_vectors_iota(int baseline, int staA_pos, int staB_pos, int staC_pos, double *BN_, double *BE_, double *BU_)
{
    float Bn[3], Be[3], Bu[3], Bd[3];


    switch (staA_pos)
    {
	case 0:
	    Bn[0] = 0.0;
	    Be[0] = 0.0;
	    Bu[0] = 0.0;
	    Bd[0] = 0.0;
	    break;
	case 5:
	    Bn[0] = 377.4742;
	    Be[0] = 328.4712;
	    Bu[0] = -0.0716;
	    Bd[0] = 500.3800;
	    break;
	case 7:
	    Bn[0] = 530.7633;
	    Be[0] = 461.8606;
	    Bu[0] = -0.1007;
	    Bd[0] = 703.5800;
	    break;
	case 10:
	    Bn[0] = 754.9485;
	    Be[0] = 656.9425;
	    Bu[0] = -0.1432;
	    Bd[0] = 1000.7600;
	    break;
	case 14:
	    Bn[0] = 1061.5265;
	    Be[0] = 923.7211;
	    Bu[0] = -0.2014;
	    Bd[0] = 1407.1600;
	    break;
	case 15:
	    Bn[0] = 1132.4227;
	    Be[0] = 985.4137;
	    Bu[0] = -0.2148;
	    Bd[0] = 1501.1400;
	    break;
	case 20:
	    Bn[0] = 1509.8969;
	    Be[0] = 1313.8849;
	    Bu[0] = -0.2865;
	    Bd[0] = 2001.5200;
	    break;
	case 21:
	    Bn[0] = 1592.2898;
	    Be[0] = 1385.5817;
	    Bu[0] = -0.3021;
	    Bd[0] = 2110.7401;
	    break;
	case 25:
	    Bn[0] = 1887.3711;
	    Be[0] = 1642.3562;
	    Bu[0] = -0.3581;
	    Bd[0] = 2501.9000;
	    break;
	case 28:
	    Bn[0] = 2123.0530;
	    Be[0] = 1847.4423;
	    Bu[0] = -0.4028;
	    Bd[0] = 2814.3201;
	    break;
	case 30:
	    Bn[0] = 2264.8454;
	    Be[0] = 1970.8274;
	    Bu[0] = -0.4297;
	    Bd[0] = 3002.2800;
	    break;
	case 35:
	    Bn[0] = 2653.8163;
	    Be[0] = 2309.3029;
	    Bu[0] = -0.5035;
	    Bd[0] = 3517.9001;
	    break;
	default:
	    printf("Invalid value for position of station A!\n");
	    break;
    };
    
    switch (staB_pos)
    {
	case 0:
	    Bn[1] = 0.0;
	    Be[1] = 0.0;
	    Bu[1] = 0.0;
	    Bd[1] = 0.0;
	    break;
	case 5:
	    Bn[1] = -328.4712;
	    Be[1] = 377.4742;
	    Bu[1] = 0.0716;
	    Bd[1] = 500.3800;
	    break;
	case 7:
	    Bn[1] = -461.8606;
	    Be[1] = 530.7633;
	    Bu[1] = 0.1007;
	    Bd[1] = 703.5800;
	    break;
	case 10:
	    Bn[1] = -656.9425;
	    Be[1] = 754.9485;
	    Bu[1] = 0.1432;
	    Bd[1] = 1000.7600;
	    break;
	case 14:
	    Bn[1] = -923.7211;
	    Be[1] = 1061.5265;
	    Bu[1] = 0.2014;
	    Bd[1] = 1407.1600;
	    break;
	case 15:
	    Bn[1] = -985.4137;
	    Be[1] = 1132.4227;
	    Bu[1] = 0.2148;
	    Bd[1] = 1501.1400;
	    break;
	default:
	    printf("Invalid value for position of station B!\n");
	    break;
    };
    
    switch (staC_pos)
    {
	case 0:
	    Bn[2] = 0.0;
	    Be[2] = 0.0;
	    Bu[2] = 0.0;
	    Bd[2] = 0.0;
	    break;
	case 5:
	    Bn[2] = 377.4742;
	    Be[2] = 328.4712;
	    Bu[2] = -0.0716;
	    Bd[2] = 500.3800;
	    break;
	case 7:
	    Bn[2] = 530.7633;
	    Be[2] = 461.8606;
	    Bu[2] = -0.1007;
	    Bd[2] = 703.5800;
	    break;
	case 10:
	    Bn[2] = 754.9485;
	    Be[2] = 656.9425;
	    Bu[2] = -0.1432;
	    Bd[2] = 1000.7600;
	    break;
	case 14:
	    Bn[2] = 1061.5265;
	    Be[2] = 923.7211;
	    Bu[2] = -0.2014;
	    Bd[2] = 1407.1600;
	    break;
	case 15:
	    Bn[2] = 1132.4227;
	    Be[2] = 985.4137;
	    Bu[2] = -0.2148;
	    Bd[2] = 1501.1400;
	    break;
	case 20:
	    Bn[2] = 1509.8969;
	    Be[2] = 1313.8849;
	    Bu[2] = -0.2865;
	    Bd[2] = 2001.5200;
	    break;
	case 21:
	    Bn[2] = 1592.2898;
	    Be[2] = 1385.5817;
	    Bu[2] = -0.3021;
	    Bd[2] = 2110.7401;
	    break;
	case 25:
	    Bn[2] = 1887.3711;
	    Be[2] = 1642.3562;
	    Bu[2] = -0.3581;
	    Bd[2] = 2501.9000;
	    break;
	case 28:
	    Bn[2] = 2123.0530;
	    Be[2] = 1847.4423;
	    Bu[2] = -0.4028;
	    Bd[2] = 2814.3201;
	    break;
	case 30:
	    Bn[2] = 2264.8454;
	    Be[2] = 1970.8274;
	    Bu[2] = -0.4297;
	    Bd[2] = 3002.2800;
	    break;
	case 35:
	    Bn[2] = 2653.8163;
	    Be[2] = 2309.3029;
	    Bu[2] = -0.5035;
	    Bd[2] = 3517.9001;
	    break;
	default:
	    printf("Invalid value for position of station C!\n");
	    break;
    };
    
    
    //*** calculate Baselines for AB, AC, BC
    if (baseline == 0) //AB
    {
	*BE_ = Be[0]-Be[1];
	*BN_ = Bn[0]-Bn[1];
	*BU_ = Bu[0]-Bu[1];
    }
    
    if (baseline == 1) //AC
    {
	*BE_ = Be[0]-Be[2];
	*BN_ = Bn[0]-Bn[2];
	*BU_ = Bu[0]-Bu[2];
    }
    
    if (baseline == 2) //BC
    {
	*BE_ = Be[1]-Be[2];
	*BN_ = Bn[1]-Bn[2];
	*BU_ = Bu[1]-Bu[2];
    }
	
    //to make it match with the header-file (u, v)'s
    *BE_ = -*BE_;
    *BN_ = -*BN_;
    *BU_ = -*BU_;
}
