/* ptracker.c */

#include <math.h>
#include <stdio.h>
#include <time.h>
#include <mathfunctions.h>
#include <nrutil.h>
#include <trackerdefines.h>
#include <trackerfunctions.h>

#define N_RUNS 1
#define CONTROL_GAIN 1.0 /* there appears to be some overshoot when correcting
						* this makes it correct __ of the error. */

/****************************************************
* This is the packet tracker callable from DAQ_Scheduler.cpp
* the returned value is 1 if a packet was found successfully, otherwise 0
* y_raw_double is a pointer to an array of doubles - the raw (normalized) data
* nl and nh are the index limits of that array, e.g. 0 and 127 or 1 and 256, etc.
* nDFT is a pointer to an integer containing the number of points used in the
* 	DFT window. It must be an odd number, and should equal the number of
* 	data points in two wavelengths of the fringe (13 is a good initial guess).
* 	It is updated by the function, so it should be kept alive for use
* 	on the next scan.
* nFringe is a pointer to an integer containing the number of samples from the
* 	center of an ideal fringe packet to where the sinc function crosses 0.
* 	Recommended setting = 35. Capability to adaptively set this within
* 	ptracker_ed() may be added, which is why it is a pointer. Alternatively,
* 	analysis external to ptracker_ed() could be used.
* thresholdRatio, a float, is the limit for comparing max(y_dft) (an internal variable) 
* 	with the dft readings away from the detected fringe packet. If this calculated
*   ratio > thresholdRatio, the returned value will be 1, indicating
* 	that a packet was found. Recommended setting = 5.0
* centerPhase is a pointer to a float containing the phase angle of the identified
* 	center of the fringe packet. -pi corresponds to the very first data point in 
* 	the scan, and +pi corresponds to the last one.
****************************************************/
int ptracker_ed(double *y_raw_double, unsigned long nl, unsigned long nh, 
				int *nDFT, int nPeriodsInDFTWindow, int nDFTSmooth, int nFringe, int templateType, 
				int useSymmetry, float thresholdRatio, float *weight, float *centerPhase)
{
	int check_n_dft = 1; /* whether to check to see if nDFT+4+2+0-2-4 is better
							* could be a #define */

	int n_dft_in, packetFound = 0;
	unsigned long i, i_max, binNumber, i_center, nFringe_unsigned;
	float *y_dft, *y_raw; /* vector, result of DFT on y_raw */
	float max_y_dft, max_y_dft_minus4, max_y_dft_minus2, max_y_dft_plus0, max_y_dft_plus2, max_y_dft_plus4;
	float *y_dft_minus4, *y_dft_minus2, *y_dft_plus0, *y_dft_plus2, *y_dft_plus4;
	float *tempVector;

#if LOGMSG_ENABLE
	/* logMsg("\n ptracker_ed() beginning\n",0,0,0,0,0,0); */
	logMsg("\n ptracker_ed inputs: y_raw_double[%i]=%i.%i,  y_raw_double[%i]=%i.%i\n",
		(int)nl, (int)floor(y_raw_double[nl]), roundFloat(1000*(float)(y_raw_double[nl]-floor(y_raw_double[nl]))), 
		(int)nh, (int)floor(y_raw_double[nh]), roundFloat(1000*(float)(y_raw_double[nh]-floor(y_raw_double[nh]))));
	logMsg("\n ptracker_ed inputs: *nDFT=%i,  *nFringe=%i,  thresholdRatio=%i\n",
		*nDFT, *nFringe, roundFloat(thresholdRatio), 0, 0, 0);
#endif

#if PRINT_DEBUG
	printf("\ny_raw_double[%u] = %f, y_raw_double[%u] = %f\n", nl, y_raw_double[nl], nh, y_raw_double[nh]);
	printf("\n*nDFT = %d, nFringe = %d\n", *nDFT, nFringe);
	printf("\nthreshold_ratio = %f, *centerPhase = %f\n", thresholdRatio, *centerPhase);
#endif

	y_raw = vector(nl, nh);
	y_dft_minus4 = vector(nl, nh);
	y_dft_minus2 = vector(nl, nh);
	y_dft_plus0 = vector(nl, nh);
	y_dft_plus2 = vector(nl, nh);
	y_dft_plus4 = vector(nl, nh);
	y_dft = vector(nl, nh);
	tempVector = vector(1, 5);

	/* convert inputs as needed */
	n_dft_in = *nDFT;
	nFringe_unsigned = (unsigned long) nFringe;

	for (i=nl;i<=nh;i++) {
		y_raw[i] = (float) y_raw_double[i];
	}

	binNumber = 1 + nPeriodsInDFTWindow; /* frequency bin to look in to find fringes. Bin n corresponds
	* to n-1 complete wavelengths (bin 1 is the DC component) */

	dftCalcs(y_raw, y_dft_plus0, &max_y_dft_plus0, nl, nh, *nDFT, binNumber, nDFTSmooth);

	if (check_n_dft) {
		dftCalcs(y_raw, y_dft_minus4, &max_y_dft_minus4, nl, nh, (*nDFT)-4, binNumber, nDFTSmooth);
		dftCalcs(y_raw, y_dft_minus2, &max_y_dft_minus2, nl, nh, (*nDFT)-2, binNumber, nDFTSmooth);
		dftCalcs(y_raw, y_dft_plus2, &max_y_dft_plus2, nl, nh, (*nDFT)+2, binNumber, nDFTSmooth);
		dftCalcs(y_raw, y_dft_plus4, &max_y_dft_plus4, nl, nh, (*nDFT)+4, binNumber, nDFTSmooth);

		maxDFTs(y_dft_minus4, y_dft_minus2, y_dft_plus0, y_dft_plus2, 
			y_dft_plus4, y_dft, nl, nh);
		/* sets y_dft to the maximum DFT result (on a point-by-point basis) 
		*   from each of the five window sizes */

		/* find the maximum DFT value from all 5 runs */
		tempVector[1] = max_y_dft_minus4;
		tempVector[2] = max_y_dft_minus2;
		tempVector[3] = max_y_dft_plus0;
		tempVector[4] = max_y_dft_plus2;
		tempVector[5] = max_y_dft_plus4;
		i_max = maximum_vector(tempVector,1,5);
		max_y_dft = tempVector[i_max]; /* the highest from all 5 runs */
		if (max_y_dft > max_y_dft_plus0*1.05) { 
			/* the peak of the DFT is higher for a different window size than 
			* the current setting, so use this window size */
			if (i_max < 3) { /* reduce n_DFT */
				*nDFT = *nDFT - 2;
			}
			else if (i_max > 3) { /* increase n_DFT */
				*nDFT = *nDFT + 2;
			}
			if (*nDFT < 7) { /* doesn't let n_DFT drop below 7 */
				*nDFT = 7;
			}

#if PRINT_ENABLE
			printf("ptracker_ed: nDFT = %d\n", *nDFT);
#endif
		}
	}
	else {
		y_dft = y_dft_plus0;
		max_y_dft = max_y_dft_plus0;
	} /* if check_n_dft */

	i_center = findInterferogramCenter(y_dft, nl, nh, nFringe_unsigned, useSymmetry, templateType);
	*centerPhase = (float) (((float)(i_center-nl) / (float)(nh-nl) * 2.0 - 1.0) *
		PI * CONTROL_GAIN);

	*weight = check_dft_ratio(y_dft, nl, nh, (unsigned long) *nDFT, i_center);
	/* check confidence in this detection. Check the ratio between the maximum
	*	y_dft value and an average of values where a fringe should not be
	*	present. The y_dft values on each end are averaged, then the lower of 
	*	these values is used to calculate the ratio. */

	if (*weight > thresholdRatio) 
		packetFound = 1;
	else /* packet was not found, so set nDFT to its original value - otherwise
		 *	nDFT could wander off to a very bad value that could not be recovered
		 *	from. */
		*nDFT = n_dft_in;

#if PRINT_ENABLE
	printf("ptracker_ed: packetFound =                                             %d\n", packetFound);
	printf("ptracker_ed: i_center =                                           %d\n", i_center);
#endif

#if PRINT_DEBUG
	print_vector(y_raw, nl, nh, "ptracker_ed: y_raw");
	print_vector(y_dft, nl, nh, "ptracker_ed: y_dft");
#endif

#if LOGMSG_ENABLE
	logMsg("\n ptracker_ed end: i_center=%i,  dft_ratio=%i.%i,  packetFound=%i\n",
		(int)i_center, (int)floor(dft_ratio), roundFloat((float)(1000*(dft_ratio-floor(dft_ratio)))), packetFound, 0, 0);
	logMsg("\n ptracker_ed end: centerPhase=%i.%i,  nDFT in=%i,  nDFT out=%i\n",
		(int)floor(*centerPhase), roundFloat((float)(1000*((*centerPhase)-floor(*centerPhase)))), n_dft_in, *nDFT, 0, 0);
#endif

	free_vector(y_raw, nl, nh);
	free_vector(y_dft_minus4, nl, nh);
	free_vector(y_dft_minus2, nl, nh);
	free_vector(y_dft_plus0, nl, nh);
	free_vector(y_dft_plus2, nl, nh);
	free_vector(y_dft_plus4, nl, nh);
	free_vector(y_dft, nl, nh);
	free_vector(tempVector, 1, 5);

	return packetFound;
}

/****************************************************
* The main function for testing ptracker_ed()
* Generates an ideal fringe packet, then calls ptracker_ed() to find the center
****************************************************/
#if VXWORKS_COMPILE
/*int logMsg(char * fmt, int arg1, int arg2, int arg3, int arg4, int arg5, int arg6); */
void ptracker_main(void) /* if you compile a main() on VxWorks, you get __eabi errors */
#else
void main(void)
#endif
{
	int i, i_run;
	int packetFound; /* returned value - 1 if packet found, 0 if not */
	int nDFTSmooth = 11; /* First step after DFT calculation is to smooth it
						 * by taking the mean across a window. This is the width
						 * of that window (total number of points) */
	int nDFT; /* number of points in the DFT window. Calculated below 
			  * based on samplesPerFringePeriod and nPeriodsInDFTWindow,
			  * then updated by ptracker_ed() */
	int nFringe = 20; /* Determines the width of the template used in convolution
					  * with the DFT result. 
					  * If a rectangular window is used
					  *   (if templateType = 1), this is the width from the center
					  *   point to the outer edge of the rectangle. So if nFringe = 20,
					  *   the window size is 41 (2*20 + 1). Recommended setting: to
					  *   cover the half-width at half maximum.
					  * If the window is rectangular with negative side lobes
					  *   (templateType = 2), this is the width from the center point
					  *   to the outer edge of each negative side lobe.
					  *   Recommended setting: so the negative side lobes are at the
					  *   edge of the main lobe of the interferogram. */

	int useSymmetry = 0; /* 0/1 - whether or not to use symmetry in post-processing
						 * the DFT result. Recommended setting = 0; */
	int templateType = 1; /* Which template shape to use for convolving the smoothed
						  * DFT signal. 1 - rectangular, 2 - rectangular with negative
						  * side lobes. Recommended setting = 1 */
	int nPtsScan2 = 128; /* number of points in a single scan */
	int samplesPerFringePeriod = 8; /* initial guess - this number is used
									* to calculate nDFT, and is automatically updated 
									* during the run. Try to set accurately, but not 
									* critical since it is adapted. */
	int nPeriodsInDFTWindow = 2; /* The number of fringe periods used to calculate the
								 * DFT. Corresponds to the bin number checked in the DFT
								 * result (bin # = nPeriodsInDFTWindow + 1) */
	float *y_raw; /* the float version of y_raw_double. ptracker_ed uses floats, but
					* DAQ_Scheduler.cpp uses doubles. So this test program accommodates
					* that. */
	double *y_raw_double; /* the input vector of normalized (A-B)/(A+B) fringe data */
	float thresholdRatio = 2.0; /* packet is considered found if the DFT ratio
						* is above this value */
	float weight; /* returned confidence measurement. Higher value means higher
					* confidence that the fringe was found. Based on DFT ratio. */
	float centerPhase; /* returned value, between -pi and pi corresponding
					   *	to the identified packet center */

#if LOGMSG_ENABLE
	logMsg("\n ptracker_main() beginning\n",0,0,0,0,0,0);
#endif
	
	/* allocate vectors using the Numerical Recipes vector design */
	y_raw_double = dvector(0,nPtsScan2-1);
	y_raw = vector(1,nPtsScan2);

	nDFT = roundOdd((float) nPeriodsInDFTWindow*samplesPerFringePeriod);
	/* rounds to the nearest odd integer, as required for the DFT window
	* this is updated periodically based on the peak of the DFT.
	* the 2.0 corresponds to the number of fringe periods that should be 
	* included in the DFT window */
#if PRINT_DEBUG
	printf("main: nDFT = %d\n", nDFT);
#endif
	
	for (i_run = 1; i_run<=N_RUNS; i_run++) {
		/* generate test data */
		generateIdealPacket(y_raw, 1, nPtsScan2);

		for (i=1;i<=nPtsScan2;i++) {
			y_raw_double[i-1] = (double) y_raw[i];
		}

		/* for (i=1;i<=10000;i++) 
		* use this for loop for compute time testing. On 2/13 it took ~9 seconds
		* for 10,000 cycles -> 0.9 ms per cycle on a 1 GHz Pentium 3 */
		packetFound = ptracker_ed(y_raw_double, 0, nPtsScan2-1, 
			&nDFT, nPeriodsInDFTWindow, nDFTSmooth, nFringe, 
			templateType, useSymmetry, thresholdRatio, &weight, &centerPhase);

#if PRINT_ENABLE
		printf("main: centerPhase = %f, packetFound =                       %d\n", centerPhase, packetFound);
		printf("----------------------------------------\n");
#endif
	} /* for N_RUNS */

	free_dvector(y_raw_double,0,nPtsScan2-1);
	free_vector(y_raw,1,nPtsScan2);
}

