/* trackerfunctions.c - supporting functions specific to ptracker.c 
	(i.e., not generic) 

	check_dft_ratio
	findInterferogramCenter
	generateIdealPacket
	maxDFTs
	meanFilter
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "mathfunctions.h"
//#include "/usr/local/recipes/src/nrutil.h"
#include "trackerdefines.h"
#include "trackerfunctions.h"

#define USE_RANDOM_PACKET_PARAMETERS 0 /* whether or not to use randomized
	* ABCDEbias when generating an ideal fringe packet. Set to 0 for comparing
	* the algorithm with MATLAB results */
double genFringeAmplitude, genFringeFrequency;
//#define genFringeAmplitude  0.6
float ADDITIVE_NOISE; /* uniformly distributed noise with this total range
	* is added to the generated scan data */

/****************************************************
* check_dft_ratio
*   check confidence in this detection. Check the ratio between the mean
*	y_dft value near i_center and the mean of values where a fringe should not be
*	present. The y_dft values on each end are averaged, then the lower of 
*	these values is used to calculate the ratio.
****************************************************/
float check_dft_ratio(float *y_dft, unsigned long nl, unsigned long nh, unsigned long nDFT, unsigned long i_center)
{
	unsigned long dft_check_nl, dft_check_nh, dft_check_size;
	float left_y_dft, right_y_dft, dft_ratio, center_y_dft;

	/* check left side */
	dft_check_size = (unsigned long) floor((nh-nl+1.0)/10.0);
	dft_check_nl = (unsigned long) floor(nl+((nDFT)/2.0));
	dft_check_nh = dft_check_nl + (unsigned long)(2.0*dft_check_size);
	left_y_dft = mean_vector(y_dft, dft_check_nl, dft_check_nh);
	/* check right side */
	dft_check_nh = nh - (dft_check_nl - nl);
	dft_check_nl = dft_check_nh - (unsigned long)(2.0*dft_check_size);
	right_y_dft = mean_vector(y_dft, dft_check_nl, dft_check_nh);
	/* check near i_center */
	dft_check_nl = IMAX(nl,i_center - dft_check_size);
	dft_check_nh = IMIN(nh,i_center + dft_check_size);
	center_y_dft = mean_vector(y_dft, dft_check_nl, dft_check_nh);

	dft_ratio = (center_y_dft / FMIN(left_y_dft, right_y_dft)) - 1;
	/* 1 subtracted here to make it more like the "weight" calculation used by
	*   Ettore's tracker: (peak-background)/background = peak/background - 1 */

#if PRINT_ENABLE
	printf("check_dft_ratio: left_y_dft = %f  right_y_dft = %f\n", left_y_dft, right_y_dft);
	printf("check_dft_ratio: center_y_dft = %f  dft_ratio = %f\n", center_y_dft, dft_ratio);
#endif

	return dft_ratio;
}

/****************************************************
* dftCalcs
*   DFT calculations done repeatedly (5 times per packet)
*   Calculates y_dft (normalized and smoothed) and the max of that
*     vector, max_y_dft
****************************************************/
void dftCalcs(float *y_raw, float *y_dft, float *max_y_dft, 
			  unsigned long nl, unsigned long nh, int nDFT, int binNumber, int nDFTSmooth)
{
	float *tempVector;

	tempVector = vector(nl, nh);

	dft_bin_nocomplex(y_raw, y_dft, nl, nh, nDFT, binNumber);
	/* calculates the (binNumber result only) DFT of y_raw, returns as y_dft */

	mult_scalar_vector(y_dft, (float) (1.0/nDFT), tempVector, nl, nh);
	/* normalizes y_dft and stores as tempVector before smoothing
	* the DFT results are divided by the bin size to allow fair comparison
	* between different nDFTs. dft_bin_nocomplex returns values exactly as 
	* they would be if calculated using the MATLAB FFT() function. */

	meanFilter(tempVector, y_dft, nl, nh, nDFTSmooth);
	/* smooths the DFT result, replacing y_dft_plus0 */

	*max_y_dft = y_dft[maximum_vector(y_dft, nl, nh)];
	/* the maximum value of the smoothed DFT result. Used to adaptively set nDFT. */

	free_vector(tempVector, nl, nh);
}

/****************************************************
* findInterferogramCenter
*
* finds the center of the interferogram, given a vector, x, that is approximately
*   the envelope of the scan data.
*
* As of May 2002, only useSymmetry==0 and templateType==1 are supported.
* useSymmetry==1 and templateType==2 was developed for the Feb 2002 
* implementation - see the code there if re-implementing
*
* Edward Wilson 1997-2002
****************************************************/
unsigned long findInterferogramCenter(float *x, unsigned long nl, unsigned long nh,
							 unsigned long nFringe, int useSymmetry, int templateType)
{
	unsigned long i_center;
	float *peakFactor;

	peakFactor = vector(nl, nh);
	meanFilter(x, peakFactor, nl, nh, 2*nFringe+1);

	i_center = maximum_vector(peakFactor, nl, nh);

#if PRINT_DEBUG
	print_vector(peakFactor, nl, nh, "findInterferogramCenter: peakFactor");
	printf("findInterferogramCenter: i_center = %d\n\n",i_center);
#endif

	free_vector(peakFactor, nl, nh);

	return i_center;
}

/****************************************************
* Generates an ideal fringe packet with pseudo-random ABCDE_bias parameters
*   - used for testing
****************************************************/
void generateIdealPacket(float *y_raw, unsigned long nl, unsigned long nh)
{
	unsigned long i;
	float *t, *ABCDE_bias, *ABCDE_bias_low, *ABCDE_bias_high;
	/* parameters of the generated fringe function
	*	y = A * sinc(B(t+C)) .* cos(D(t+E)) + bias 
	*	the _low and _high are the limits for the random number generation */
	float frand;

	ABCDE_bias = vector(1,6);
	ABCDE_bias_low = vector(1,6);
	ABCDE_bias_high = vector(1,6);
	t = vector(nl, nh);

 	ABCDE_bias_low[1] = (float) 0.2;
	ABCDE_bias_low[2] = (float) 0.025;
	ABCDE_bias_low[3] = -((float) (nh*1.2));
	ABCDE_bias_low[4] = (float) 0.9;
	ABCDE_bias_low[5] = ABCDE_bias_low[3];
	ABCDE_bias_low[6] = (float) -1.0;

 	ABCDE_bias_high[1] = (float) genFringeAmplitude;
	ABCDE_bias_high[2] = (float) 0.03;
	ABCDE_bias_high[3] =  ((float) (nh*0.2));
	ABCDE_bias_high[4] = (float) 1.1;
	ABCDE_bias_high[5] = ABCDE_bias_high[3];
	ABCDE_bias_high[6] = (float) 1.0;

	/* Seed the random-number generator with current time so that
	*	the numbers will be different every time we run.
	* rand() will return an integer between 0 and RAND_MAX, which
	*	has a pseudo-random uniform distribution.
	* To get a random float between 0.0 and 1.0, use
	*		x = rand()/(RAND_MAX+1.0);
	* According to Numerical Recipes, this is a relatively bad
	*	algorithm, so if important, use a NRinC generator */
/*	srand((unsigned)time(NULL)); */

	for (i=1; i<=6; i++) {
		frand = (float) (rand() / (RAND_MAX+1.0));
		ABCDE_bias[i] = ABCDE_bias_low[i] + frand * (ABCDE_bias_high[i]-ABCDE_bias_low[i]);
	}

#if !USE_RANDOM_PACKET_PARAMETERS /* use this for comparing with matlab */
	ABCDE_bias[1] = (float) genFringeAmplitude;
	ABCDE_bias[2] = (float) 0.027;
	ABCDE_bias[3] = (float) -110.0;
	ABCDE_bias[4] = (float) 1.0/40.0*genFringeFrequency;
	ABCDE_bias[5] = (float) -110.0;
	ABCDE_bias[6] = (float) 0.3;
#endif

#if PRINT_ENABLE
	printf("\ngenerateIdealPacket: Generated fringe parameters:\n");
	print_vector(ABCDE_bias, 1, 6, "generateIdealPacket: ABCDE_bias");
#endif

	for (i=nl; i<=nh; i++) {
        t[i] = (float) i;
		
		frand = (float) ((rand() / (RAND_MAX+1.0) - 0.5) * ADDITIVE_NOISE);
		y_raw[i] = (float) (ABCDE_bias[1] * sinc(ABCDE_bias[2]*(t[i]+ABCDE_bias[3])) * cos(ABCDE_bias[4]*(t[i] + ABCDE_bias[5])) + ABCDE_bias[6] + frand);
	}

	free_vector(ABCDE_bias,1,6);
	free_vector(ABCDE_bias_low,1,6);
	free_vector(ABCDE_bias_high,1,6);
	free_vector(t, nl, nh);
}

/****************************************************
* maxDFTs
*   given 5 vectors, return the maximum vector (taken element-by-element)
****************************************************/
void maxDFTs(float *x1, float *x2, float *x3, float *x4, float *x5, 
			 float *y, unsigned long nl, unsigned long nh)
{
	copy_vector(x1, y, nl, nh);
	maximum_vectors(y, x2, nl, nh);
	maximum_vectors(y, x3, nl, nh);
	maximum_vectors(y, x4, nl, nh);
	maximum_vectors(y, x5, nl, nh);
}

/****************************************************
* meanFilter
*
*
* filters a vector, x, by passing a window of width N samples over it and
*   taking the mean of the window as the filter output.
*
* this is an acauasal filter since the window uses future and past values
*   of x to calculate the output y. averagingFilter() is similar - but
*   it is a causal filter, using the preceding N values for the window,
*   and is implemented using the MATLAB built-in filter() command.
*
* N must be an odd number so the window has a single central sample. If 
*   given as an even number, it is incremented by 1.
*
* Edward Wilson 2002
*
* passes a template over the data that looks like:
*   +++++++++++++++++++
*   1                 2
* this is supposed to result in a high signal when the template is 
*   centered over the peak. Result is peakFactor. The dimensions of
*   the template are defined by nFringe
* with the center of the template at i,
*   point 1 is at i-(N-1)/2
*   point 2 is at i+(N-1)/2
****************************************************/
void meanFilter(float *x, float *y, unsigned long nl, unsigned long nh,
							 unsigned long N)
{
	unsigned long N2, npts, i, nWindow;

	if (remainder((float) N, 2.0) == 0) { /* N is even */
		N++; /* make it odd */
	}
	N2 = (N-1)/2;

	npts = nh - nl + 1;

	if (N == 1) { /* nothing to do in this case */
		copy_vector(x, y, nl, nh);
	} 
	else {
		y[nl] = mean_vector(x,nl,nl+N2);
		for (i=nl+1; i<=nl+N2; i++) { /* points until window is full (N-wide) */
			y[i] = (float) (y[i-1]*(i+N2-nl)/(i+N2+1.0-nl) + x[i+N2]/(i+N2+1.0-nl));
		}
		for (i=nl+N2+1; i<=nh-N2; i++) { /* middle points */
			y[i] = y[i-1] + (x[i+N2]-x[i-N2-1])/((float) N);
		}
		for (i=nh-N2+1; i<=nh; i++) { /* points at end with less than a full window */
			nWindow = nh-i+1+N2;
			y[i] = (y[i-1] - x[i-N2-1]/((float) nWindow+1)) * (float) ((nWindow+1.0)/((float) nWindow));
		}
	}
}
